import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/*
	BattleFrame.java
  	(c)1999-2008 SSC Development Team
	Functions:
	-Managing the Game GUI and input as a whole.
	Allowed Dependencies: BattlePanel.java
*/

public class BattleFrame 	extends JFrame
						implements 	ActionListener, KeyListener,
									WindowFocusListener
{
	Container c = getContentPane();

	Timer timer;
  private JTextField prompt;

	BattlePanel screen;
	MeleeSequencePanel battle_anim;
	RangedSequencePanel ranged_anim;
  SpellSequencePanel spell_anim;

  /* Value overview of directionals:
   *
   * 0 not moving up
   * 1 moving up, and the corresponding arrow key is still being pressed.
   * 2 moving up, but the key has been released.
   * 3 arrow key has been pressed, but other movement directions
   *   override this one
   */

	private int up = 0;
	private int down = 0;
	private int left = 0;
	private int right = 0;

  /*equals 1 if particular button was once pressed and not released yet.*/
	private int opup = 0;
	private int opdown = 0;
	private int opleft = 0;
	private int opright = 0;

	private static int mode = 0;
	public static int Mode() { return(mode); }
	/* Mode variable
	 * 0   = overland walking
	 * 1   = standard menu active
	 * 2   = select your target
	 * 5   = attack
	 * 6   = status check
	 * 7   = End of turn (text) event
	 */
	private static int bm_option = 0; //battle mode option [multi-purpose]
	public static int Bm_option() { return(bm_option); }
	/* Menu option (if mode == 1)
	 * 0 = attack
	 * 1 = ability
	 * 2 = item
	 * 3 = stay
	 * 10-13 = spell options
	 * Target selection var (if mode == 2)
	 */
  private static int spell_lv_choice = 1; //spell level storage var.
  public static int SL_choice() { return(spell_lv_choice); }

  private static int atk_type = 0;
  public static int Atk_type() { return(atk_type); }
  /* Type of attack taken
   * 0 = normal attack
   * 1 = ranged attack
   * 2 = spell 0
   * 3 = spell 1
   * 4 = spell 2
   * 5 = spell 3
   */

	private static int[] targets; /* Targets within range */
	public static int getTarget() {
	  if(bm_option >= targets.length) {
	    bm_option = 0;
	  }
	  return targets[bm_option];
	}

	/* Constructor */
	public BattleFrame(WalkMap cur_map)
	{
	  this.setVisible(false);

	  screen = new BattlePanel(cur_map,
	  Buffer.Player().x,
	  Buffer.Player().y);

    Buffer.getCurMap().placeMoveObs(Buffer.Player());

	  timer = new Timer(Buffer.TimerDelay, this); //Timer delay.TWEAKABLE
	  prompt = new JTextField("");

    screen.setBounds(0,0,(int)(360*Buffer.ScaleRatio),(int)(240*Buffer.ScaleRatio));

	  c.setLayout(null);

	  c.add(screen);
	  c.add(prompt);

	  prompt.addKeyListener(this);
    this.addWindowFocusListener(this);

	  this.setVisible(true);

	  startAnimation();
	}

  /* initialization after map switches */
	public void init() {
    screen = new BattlePanel(Buffer.getCurMap(),Buffer.Player().x,Buffer.Player().y);
    Buffer.getCurMap().placeMoveObs(Buffer.Player());

    Timer timer = new Timer(Buffer.TimerDelay, this); //Timer delay. TWEAKABLE
    JTextField prompt = new JTextField("");

    screen.setBounds(0,0,(int)(360*Buffer.ScaleRatio),(int)(240*Buffer.ScaleRatio));

    c.setLayout(null);

    c.add(screen);
    c.add(prompt);

    JukeBox.Init();

    prompt.addKeyListener(this);
    this.addWindowFocusListener(this);

    this.setVisible(true);

    JukeBox.PlayWalkTune();
    startAnimation();
	}

  /* Switch to a different map! */
  public void SwitchMap(String mapname)
  {
    stopAnimation();
    JukeBox.StopAll();
    this.setVisible(false);

    c.remove(screen);
    c.remove(prompt);

    Lib_Map.initMap(0,this,mapname);

    Buffer.firstPlayer();
  }

	public synchronized void startAnimation() {
    timer.start();
  }


  public synchronized void stopAnimation() {
    timer.stop();
  }

  private int anim_delay = Buffer.AnimDelay; //Animation delay per frame. TWEAKABLE
  private int anim_iterator = 0;

	/*
	 * DRAWING ROUTINE
	 */

  private String movestring;
  private char curmove;

  /* Resets all movement stuff */
  public void resetMovement() {
    up = 0;
    down = 0;
    left = 0;
    right = 0;
    opup = 0;
    opdown = 0;
    opleft = 0;
    opright = 0;
  }

  public void actionPerformed(ActionEvent e) {

    int newdirection = 0;

    /* A.I. targeting execution code!!! */
    if(Buffer.Player().ai == true && mode == 2) {
      if(anim_iterator == anim_delay-1) {
        c.remove(screen);

        if(Buffer.Player().range > 2 && atk_type == 0) { atk_type = 1; }

        if(atk_type == 0) {
          battle_anim = new MeleeSequencePanel(Buffer.Player(),Buffer.getEnt(targets[0]));
          battle_anim.setBounds(0,0,(int)(360*Buffer.ScaleRatio),(int)(240*Buffer.ScaleRatio));

          /* TODO: Stop local animation loop? */
          c.add(battle_anim);
        }
        else if(atk_type == 1) {
          ranged_anim = new RangedSequencePanel(Buffer.Player(),Buffer.getEnt(targets[0]));
          ranged_anim.setBounds(0,0,360*(int)(Buffer.ScaleRatio),(int)(240*Buffer.ScaleRatio));

          /* TODO: Stop local animation loop? */
          c.add(ranged_anim);
        }
        this.paint(this.getGraphics());
        mode = 5;
        JukeBox.PlayBattleTune();
      }
    }

    /* A.I. movescript execution code!!! */
    if(Buffer.Player().ai == true && mode == 0) {

      // Clear old moves
      if(curmove != ' ' && Buffer.Player().y%24 == 0 && Buffer.Player().x%24 == 0
         && left != 3 && right != 3 && up != 3 && down != 3) {
        curmove = ' ';
        up = 0;
        down = 0;
        left = 0;
        right = 0;
        newdirection = 1;
      }

      // Grab next move from the movescript!
      if(curmove == ' ') {
        if(movestring.charAt(0) == 'r') {
          right   = 1;
          opright = 1;
          left    = 0;
          Buffer.Player().f = 2 + Buffer.anim_toggle;
          curmove = 'r';
          movestring = movestring.substring(1);
          //System.out.println("r:"+movestring);
        }
        else if(movestring.charAt(0) == 'u') {
          up      = 1;
          opup    = 1;
          down    = 0;
          Buffer.Player().f = 0 + Buffer.anim_toggle;
          curmove = 'u';
          movestring = movestring.substring(1);
          //System.out.println("u:"+movestring);
        }
        else if(movestring.charAt(0) == 'd') {
          down    = 1;
          opdown  = 1;
          up      = 0;
          Buffer.Player().f = 4 + Buffer.anim_toggle;
          curmove = 'd';
          movestring = movestring.substring(1);
          //System.out.println("d:"+movestring);
        }
        else if(movestring.charAt(0) == 'l') {
          left    = 1;
          opleft  = 1;
          right   = 0;
          Buffer.Player().f = 6 + Buffer.anim_toggle;
          curmove = 'l';
          movestring = movestring.substring(1);
          //System.out.println("l:"+movestring);
        }
        else {
        /* Attempt an attack sequence! */
          targets = Buffer.getAdjacentEnts(true);
          if(targets.length>0) { mode = 2; atk_type = 0; } //to targeting mode
          else {
            //code for going to next player
            Buffer.getCurMap().setValue(Buffer.Player().x/24,Buffer.Player().y/24,5,(Buffer.playerIndex()+1));
            Buffer.checkAlive();

            movestring = Buffer.nextPlayer();
            resetMovement();    //is this causing the problem of continued movement?

            screen.Update(Buffer.Player().x,Buffer.Player().y);
            mode = 0;
          }
        }
      }
    }


		//tile & sprite animation
    anim_iterator++;
    if(anim_iterator>=anim_delay) {
      anim_iterator = 0;
      Buffer.getCurMap().updateMap();
      Buffer.anim_toggle = 1 - Buffer.anim_toggle;

      /* Entity animation */
      if(Buffer.AnimateAllEnts == true) { //Animate all
        for(int i=0; i<Buffer.numEnts(); i++) {
          Buffer.getEnt(i).f = Buffer.getEnt(i).f - (Buffer.getEnt(i).f%2) + Buffer.anim_toggle;
        }
      }
      else { //Animate player only
        Buffer.Player().f = Buffer.Player().f - (Buffer.Player().f%2) + Buffer.anim_toggle;
      }
    }


    //block next movement if obstructed!

    int ptx = Buffer.Player().x/24; //player tx
    int pty = Buffer.Player().y/24; //player ty
    int fac = Buffer.Player().faction;
    int spritenum = 0;

      if(Buffer.Player().y%24 == 0) {
        spritenum = Buffer.getCurMap().getValue(ptx,pty-1,5);
			  if((Buffer.getCurMap().getValue(ptx,pty-1,3) == 1 ||
			    (spritenum > 0 && Buffer.getEnt(spritenum-1).faction != fac)) && up == 1) {
				  up = 0;
				  opup = 1;
          newdirection = 1;
          //if(Buffer.Player().f/2 != 0) { Buffer.Player().f = 0 + Buffer.anim_toggle; }
			  }
        spritenum = Buffer.getCurMap().getValue(ptx,pty+1,5);
			  if((Buffer.getCurMap().getValue(ptx,pty+1,3) == 1 ||
          (spritenum > 0 && Buffer.getEnt(spritenum-1).faction != fac)) && down == 1) {
				  down = 0;
				  opdown = 1;
          newdirection = 1;
          //if(Buffer.Player().f/2 != 2) { Buffer.Player().f = 4 + Buffer.anim_toggle; }
			  }
		  }
  		if(Buffer.Player().x%24 == 0) {
        spritenum = Buffer.getCurMap().getValue(ptx+1,pty,5);
  			if((Buffer.getCurMap().getValue(ptx+1,pty,3) == 1 ||
          (spritenum > 0 && Buffer.getEnt(spritenum-1).faction != fac)) && right == 1) {
  				right = 0;
  				opright = 1;
          newdirection = 1;
          //if(Buffer.Player().f/2 != 1) { Buffer.Player().f = 2 + Buffer.anim_toggle; }
  		  }
        spritenum = Buffer.getCurMap().getValue(ptx-1,pty,5);
  			if((Buffer.getCurMap().getValue(ptx-1,pty,3) == 1 ||
          (spritenum > 0 && Buffer.getEnt(spritenum-1).faction != fac)) && left == 1) {
  				left = 0;
  				opleft = 1;
          newdirection = 1;
          //if(Buffer.Player().f/2 != 3) { Buffer.Player().f = 6 + Buffer.anim_toggle; }
  		  }
      }


    //actual movement
    int d = Buffer.PixelSkip;

    if(up==1 && Buffer.Player().y>1 )
    	{
    	Buffer.Player().y -= d; screen.Update(Buffer.Player().x,Buffer.Player().y);
    	//if(Buffer.Player().f/2 != 0) { Buffer.Player().f = 0 + Buffer.anim_toggle; }
    	}
    else if(down==1 && Buffer.Player().y < (Buffer.getCurMap().getHeight()*24)-24)
    	{
    	Buffer.Player().y += d; screen.Update(Buffer.Player().x,Buffer.Player().y);
    	//if(Buffer.Player().f/2 != 2) { Buffer.Player().f = 4 + Buffer.anim_toggle; }
    	}
    else if(left==1 && Buffer.Player().x>1)
    	{
    	Buffer.Player().x -= d; screen.Update(Buffer.Player().x,Buffer.Player().y);
    	//if(Buffer.Player().f/2 != 3) { Buffer.Player().f = 6 + Buffer.anim_toggle; }
    	}
    else if(right==1 && Buffer.Player().x < (Buffer.getCurMap().getWidth()*24)-24)
    	{
    	Buffer.Player().x += d; screen.Update(Buffer.Player().x,Buffer.Player().y);
    	//if(Buffer.Player().f/2 != 1) { Buffer.Player().f = 2 + Buffer.anim_toggle; }
    	}



    //finish old movement!


    else if(up==2 && Buffer.Player().y>1)
      {
      if(Buffer.Player().y%24 !=0) { Buffer.Player().y -= d; screen.Update(Buffer.Player().x,Buffer.Player().y); }
      else {
      up = 0;
      newdirection = 1;
      }
    }

    else if(down==2 && Buffer.Player().y < (Buffer.getCurMap().getHeight()*24)-24)
      {
      if(Buffer.Player().y%24 !=0) { Buffer.Player().y += d; screen.Update(Buffer.Player().x,Buffer.Player().y); }
      else {
      down = 0;
      newdirection = 1;
      }
    }

    else if(left==2 && Buffer.Player().x>1)
      {
      if(Buffer.Player().x%24 !=0) { Buffer.Player().x -= d; screen.Update(Buffer.Player().x,Buffer.Player().y); }
      else {
      left = 0;
      newdirection = 1;
      }
    }

    else if(right==2 && Buffer.Player().x < (Buffer.getCurMap().getWidth()*24)-24)
      {
      if(Buffer.Player().x%24 !=0) { Buffer.Player().x += d; screen.Update(Buffer.Player().x,Buffer.Player().y); }
      else {
      right = 0;
      newdirection = 1;
      }
    }

    else //if((up+1)/2 != 1 && (down+1)/2 != 1 && (left+1)/2 != 1 && (right+1)/2 != 1)
    {
     newdirection = 1;
    }

    if(newdirection == 1) {
      if     (up    == 3) { up    = 1; }
      else if(down  == 3) { down  = 1; }
      else if(left  == 3) { left  = 1; }
      else if(right == 3) { right = 1; }
    }

    //Map repaint
  	screen.repaint();
  }

  /*
   * KEYBOARD BUTTON HANDLING
   */

	public void keyPressed(KeyEvent e) {

    if(Buffer.Player().ai == true && mode<5) {
      return;
    }

		/* key becomes "VK_<something>" */
		int key = e.getKeyCode();

		/* Arrow keys initiate movement by queueing direction*/

    /* Overland walking controls */
		if(mode == 0) {
  		if(key == e.VK_UP && up != 1) {
  		/* "up != 1" prevents from queueing movement while the sprite is already
  		 * moving in the corresponding direction */
          up      = 3;
          opup    = 1;
          down    = 0;
          Buffer.Player().f = 0 + Buffer.anim_toggle;
  			}
  		else if(key == e.VK_DOWN && down != 1) {
  				down    = 3;
          opdown  = 1;
  				up      = 0;
          Buffer.Player().f = 4 + Buffer.anim_toggle;
  			}
  		else if(key == e.VK_LEFT && left != 1) {
  				left    = 3;
          opleft  = 1;
  				right   = 0;
          Buffer.Player().f = 6 + Buffer.anim_toggle;
  			}
  		else if(key == e.VK_RIGHT && right != 1) {
  				right   = 3;
  				opright = 1;
  				left    = 0;
          Buffer.Player().f = 2 + Buffer.anim_toggle;
  			}
  		else if(key == e.VK_N) {
  		  SwitchMap(Buffer.nextMap);
  		}
	  }

		/* Battle Menu Controls */
		else if(mode == 1) {
      if(key == e.VK_UP) {
        if(bm_option < 4) bm_option = 0;
        else if(bm_option > 10) {
          bm_option--;
          spell_lv_choice = Buffer.Player().spell[bm_option-10].level;
        }
      }
      else if(key == e.VK_DOWN) {
        if(bm_option < 4) bm_option = 3;
        else if(bm_option > 9 && bm_option < 13) {
          if(Buffer.Player().spell[bm_option-9] != null) {
            bm_option++;
            spell_lv_choice = Buffer.Player().spell[bm_option-10].level;
          }
        }
      }
      else if(key == e.VK_LEFT) {
        if(bm_option < 4) bm_option = 1;
        else if(bm_option > 9 && bm_option < 13) {
           if(spell_lv_choice > 1) {
             spell_lv_choice--;
             //System.out.println("--:" +spell_lv_choice);
           }
        }
      }
      else if(key == e.VK_RIGHT) {
        if(bm_option < 4) bm_option = 2;
        else if(bm_option > 9 && bm_option < 13) {
           if(spell_lv_choice < Buffer.Player().spell[bm_option-10].level) {
             spell_lv_choice++;
             //System.out.println("++:"+spell_lv_choice);
           }
        }
      }
    }

    /* Target Selection Controls */
    else if(mode == 2) {
      if(key == e.VK_UP || key == e.VK_LEFT) {
        if(bm_option>0) { bm_option--; }
        else { bm_option = targets.length-1; }
      }
      else if(key == e.VK_DOWN || key == e.VK_RIGHT) {
        bm_option++;
        bm_option = bm_option%targets.length;
      }
		}

    /*
     * SSC is not a browser, but F5 does invoke a screen refresh.
     * Just in case really ;)
     */
		if(key == e.VK_F5)
		{
		  this.paint(this.getGraphics());
	  }
		//if(key == e.VK_ENTER)
		else if(key == e.VK_A || key == e.VK_D)
		{
      if(mode == 0) {
        if(Buffer.getCurMap().getValue(Buffer.Player().x/24,Buffer.Player().y/24,5) == 0) {
          /* TODO: Finish remaining movement */
		      mode = 1;
		      bm_option = 0;
		      }
		    }

		  /* Mode 1 selection (Attack/Ability/Item/Stay menu) */

		  else if(mode == 1 && bm_option == 0) { /* Select Attack */
		    targets = Buffer.getAdjacentEnts(true);
		    if(targets.length>0) { mode = 2; atk_type = 0; } //to targeting mode
		    else { mode = 0; } //go back to mode 0 if nobody is in range
		  }
		  else if(mode == 1 && bm_option == 1) { /* Select Spell/Ability */
        bm_option = 0;
        if(Buffer.Player().spell[0] != null) {
          bm_option = 10;
          spell_lv_choice = Buffer.Player().spell[0].level;
        }
        else {
          mode = 0;
          //if(Buffer.Player().spell0 == null) System.out.println("spell0 is null?");
        }
		  }
      else if(mode == 1 && bm_option == 3 && Buffer.DetectTalk() > 0 ) {
        /* TODO: Add talk events. */
        mode = 7;
      }
      else if(mode == 1 && bm_option>9 && bm_option<14) {
        targets = Buffer.getAdjacentEnts((new Spell(Buffer.Player().spell[bm_option-10].shortname,spell_lv_choice)));
        if(targets.length>0 && Buffer.Player().curmp>(new Spell(Buffer.Player().spell[bm_option-10].shortname,spell_lv_choice)).mp_cost)
        //Buffer.Player().spell[bm_option-10].mp_cost)
          { mode = 2; atk_type = bm_option - 8; bm_option = 0; } //to targeting mode
      }

		  else if(mode == 2) { /* Target selected */
        c.remove(screen);

        if(Buffer.Player().range > 2 && atk_type == 0) { atk_type = 1; }

        if(atk_type == 0) {
          battle_anim = new MeleeSequencePanel(Buffer.Player(),Buffer.getEnt(getTarget()));
          battle_anim.setBounds(0,0,(int)(360*Buffer.ScaleRatio),(int)(240*Buffer.ScaleRatio));

          /* TODO: Stop local animation loop? */
          c.add(battle_anim);
        }

        else if(atk_type == 1) {
          ranged_anim = new RangedSequencePanel(Buffer.Player(),Buffer.getEnt(getTarget()));
          ranged_anim.setBounds(0,0,360*(int)(Buffer.ScaleRatio),(int)(240*Buffer.ScaleRatio));

          /* TODO: Stop local animation loop? */
          c.add(ranged_anim);
        }

        else if(atk_type > 1 && atk_type < 6) {
          spell_anim = new SpellSequencePanel(Buffer.Player(),Buffer.getEnt(getTarget()),new Spell(Buffer.Player().spell[atk_type-2].shortname,spell_lv_choice));
          spell_anim.setBounds(0,0,360*(int)(Buffer.ScaleRatio),(int)(240*Buffer.ScaleRatio));

          /* TODO: Stop local animation loop? */
          c.add(spell_anim);
        }

        this.paint(this.getGraphics());
        mode = 5;
        JukeBox.PlayBattleTune();
      }

      else if(mode == 5) {  //Attack animation is ongoing or ending
        if(atk_type == 0) {
          if(battle_anim.Force() == false) {

            c.remove(battle_anim);
            c.add(screen);

            this.paint(this.getGraphics());

                //code for going to next player
            Buffer.getCurMap().setValue(Buffer.Player().x/24,Buffer.Player().y/24,5,(Buffer.playerIndex()+1));
            Buffer.checkAlive();

            movestring = Buffer.nextPlayer();
            resetMovement();

            screen.Update(Buffer.Player().x,Buffer.Player().y);
            mode = 0;
            JukeBox.PlayWalkTune();

          }
        }
        else if(atk_type == 1) {
          if(ranged_anim.Force() == false) {
            atk_type = 0;
            c.remove(ranged_anim);
            c.add(screen);

            this.paint(this.getGraphics());

            //code for going to next player
            Buffer.getCurMap().setValue(Buffer.Player().x/24,Buffer.Player().y/24,5,(Buffer.playerIndex()+1));
            Buffer.checkAlive();

            movestring = Buffer.nextPlayer();
            resetMovement();

            screen.Update(Buffer.Player().x,Buffer.Player().y);
            mode = 0;
            JukeBox.PlayWalkTune();
          }
        }
        else if(atk_type > 1 && atk_type < 6) {
          if(spell_anim.Force() == false) {
            atk_type = 0;
            c.remove(spell_anim);
            c.add(screen);

            this.paint(this.getGraphics());

            //code for going to next player
            Buffer.getCurMap().setValue(Buffer.Player().x/24,Buffer.Player().y/24,5,(Buffer.playerIndex()+1));
            Buffer.checkAlive();

            movestring = Buffer.nextPlayer();
            resetMovement();

            screen.Update(Buffer.Player().x,Buffer.Player().y);
            mode = 0;
            JukeBox.PlayWalkTune();
          }
        }
      }
		  else if(mode == 6) {
		    mode = 0;
		  }
		  else { //"Stay" option selected (or coming out of mode 7)

        if(mode == 7) {

          /* BATTLE SCRIPTED TALK EVENTS SECTION  Specific for Mode 7! */

          /* implement join event after talk */
          if(Buffer.getEnt(Buffer.DetectTalk()).respevent.equals("join")) {
            Buffer.getEnt(Buffer.DetectTalk()).faction = Buffer.Player().faction;
            Buffer.getEnt(Buffer.DetectTalk()).ai = false;
          }
          /* in this case. combat ends right after the talk */
          else if(Buffer.getEnt(Buffer.DetectTalk()).respevent.equals("end combat")) {
            SwitchMap(Buffer.nextMap);
          }
        }

		    //code for going to next player
        Buffer.getCurMap().setValue(Buffer.Player().x/24,Buffer.Player().y/24,5,(Buffer.playerIndex()+1));

		    /* Checking if faction 1 and faction 2 both still have living members*/
		    boolean f1lives = false;
		    boolean f2lives = false;

		    for(int i=0; i<Buffer.numEnts(); i++) {
		      if(Buffer.getEnt(i).faction == 1) {
		        f1lives = true;
		      }
          if(Buffer.getEnt(i).faction > 1) {
            f2lives = true;
          }
		    }

		    /* Factions are there? Go to next turn! */
		    if(f1lives && f2lives) {
		      movestring = Buffer.nextPlayer();


		    }
		    /* 1 Faction is gone? */  //WHAT ABOUT IF FACTION 2 IS GONE? ie.. you lose
		    else {
		      //go directly to next map for now...
		      //FIRST REJUVINATE HP AND MP on force(and auto revive)
				rejuvinateAll();
		      //END REJUVINATION/autorevival

          	  SwitchMap(Buffer.nextMap);
		    }

        screen.Update(Buffer.Player().x,Buffer.Player().y);
        mode = 0;
      }
		}
		else if(key == e.VK_S)
		{
		  if(mode == 0) { mode = 6; }
		  else if(mode == 1 && bm_option < 10) { mode = 0; }
      else if(mode == 1 && bm_option > 9) { bm_option -= 10; }
		  else if(mode == 2) { mode = 1; }
		  else if(mode == 6) { mode = 0; }
		}
		else if(key == e.VK_F && mode == 0)
		{
      Buffer.Player().f = (Buffer.Player().f + 2)%8;
		}
	}

	public void keyTyped(KeyEvent e) { }
	public void keyReleased(KeyEvent e) {

		int key = e.getKeyCode();

		//Releasing the arrow keys stops any future movement in that direction,
		//but finishes the current move to the next tile
		if(mode == 0) {
  		if(key == e.VK_UP) {
  			up = 2;
  			opup = 0;
  			if     (opdown  == 1) { down = 1; up    = 0; }
        else if(opright == 1) { left = 0; right = 3; }
        else if(opleft  == 1) { left = 3; right = 0; }
  		}

  		if(key == e.VK_DOWN) {
  			down = 2;
  			opdown = 0;
  			if     (opup    == 1) { down = 0; up    = 1; }
        else if(opright == 1) { left = 0; right = 3; }
        else if(opleft  == 1) { left = 3; right = 0; }
  		}

  		if(key == e.VK_LEFT) {
  			left = 2;
  			opleft = 0;
  			if     (opright == 1) { left = 0; right = 1; }
        else if(opup    == 1) { down = 0; up    = 3; }
        else if(opdown  == 1) { down = 3; up    = 0; }
  		}

  		if(key == e.VK_RIGHT) {
  			right = 2;
  			opright = 0;
  			if     (opleft  == 1) { left = 1; right = 0; }
        else if(opup    == 1) { down = 0; up    = 3; }
        else if(opdown  == 1) { down = 3; up    = 0; }
  		}
		}
	}

	public void rejuvinateAll() {
		//cycle through team members, make hp and mp = max.
		for(int i=0;i<Buffer.Party.size();i++){
			Buffer.Party.elementAt(i).curhp=Buffer.Party.elementAt(i).maxhp;
			Buffer.Party.elementAt(i).curmp=Buffer.Party.elementAt(i).maxmp;
		}

	}

 	public void Refocus()
 	{
 		prompt.requestFocus();
 	}

 	/*Start animation routine on focus*/
	public void windowGainedFocus(WindowEvent e)
	{
	  timer.start();
	}

  /*Halt animation routine on unfocus*/
  public void windowLostFocus(WindowEvent e) { timer.stop(); }
}
