import java.util.*; import java.awt.image.*; import java.awt.*;

/*
	Buffer.java
	(c)1999-2008 SSC Development Team
	Function: Buffering tile images to increase performance
	Allowed Dependencies: All

	- Buffer.java effectively "buffers" the tile images. The source of the tile
	images are stored in Tile.java. The images themselves are not stored there
	because it prevents the Tiles from being serializable, making it much harder
	to write them to a file properly.
	- Buffer.java also saves some global variables like the current map and the
	currently selected layer for reference by other classes.
*/

public class Buffer
{
	/* Game/SMEE Settings! */
	public static boolean FullDraw = true; //SMEE
	public static boolean AnimateAllEnts = true; //SSC
	public static int TimerDelay = 13; //SSC Timer Delay
  public static int AnimDelay = 20; //SSC Animation Delay
  public static int PixelSkip = 2; //SSC Pixel Skip
  public static boolean MusicOn = false; //Music disabled
  public static double ScaleRatio = 1.0; //Graphics scaling ratio
  public static int Intro = 1; //Whether intro is played or not.

	/* General Stuff */

	private static WalkMap cur_map; //current map
	public static Properties cur_script; // current script

	public static void setCurMap(Object map) { cur_map = (WalkMap) map; }
	public static WalkMap getCurMap() { return cur_map; }

	public static boolean visibleRoofs = true;


	/* Game Stuff */

	//scripting
	public static String nextMap;
	public static String currentMusic;
	public static String entityFile;
	public static String mapFile;
	public static String frameFile;
	public static int map_type;
	public static int StartX, StartY; //Party / Main char starting locations

	//movie stuff
	public static int[] movieTicks;
	public static String[] movieFrames;

	//movement range matrix offsets
  public static int MMorigX, MMorigY;

	private static int CX, CY; //Camera X and Y offset in pixels

	public static int  getCX()      { return CX; }
	public static void setCX(int x) { CX = x;    }

	public static int  getCY() { return CY; }
	public static void setCY(int y) { CY = y;    }

	public static int anim_toggle = 0; //TODO: Make private?

  private static int player; /* Currently controlled entity */
  private static Vector<Entity> ents = new Vector<Entity>();

  /* Party Composition */
  public static Vector<Entity> Party = new Vector<Entity>();

  /*Item Composition*/
  public static Vector<Items> items = new Vector<Items>();
  public static int gold; //have to put it somewhere.

  /*Shop Composition*/
  public static Vector<Shops> shop = new Vector<Shops>();

  public static Font normaltxt, italictxt, smalltxt;

    //resets the whole entity vector
	  public static void clearAllEnts() {
      ents = new Vector<Entity>();
	  }

    //resets the entity vector to contain just the party.
    public static void resetToPartyOnly() {
      ents = new Vector<Entity>();

      for(int i = 0; i < Party.size(); i++) {
        ents.addElement(Party.elementAt(i));
      }
    }

    /* Find a character in the party file by name */
    public static int findChar(String name) {
      for(int i = 0; i<Party.size(); i++) {
        if(Party.elementAt(i).name.equals(name)) {
          return i;
        }
      }
      return(-1);
    }


	  //returns amount of sprites currently loaded
    public static int numEnts() {
    	return ents.size();
    }

    //checks if entity is alive, kills it otherwise
    public static void checkAlive() {
      for(int i=0;i<ents.size();i++) {
        checkAlive(i);
      }
    }

    public static void checkAlive(int k) {
      if(getEnt(k).curhp < 1) {

        cur_map.setValue(getEnt(k).x/24 ,getEnt(k).y/24 ,5 ,0);
        ents.removeElementAt(k);

        //clean entity layer!
        for(int i = 0; i<ents.size(); i++) {
          if(i != player) {
            cur_map.setValue(getEnt(i).x/24 ,getEnt(i).y/24 ,5 ,i+1);
          }
        }

      }
    }

    //gets Sprite at given index
    public static Entity getEnt(int index) {
    	return (Entity) ents.elementAt(index);
    }

    //gets adjacent Sprites who wish to talk ;)
    public static int DetectTalk() {
      int[] targets = new int[numEnts()];

      Entity e = Player();

      /* In Battle */
      if(map_type == 0) {
        for(int i = 0; i < targets.length; i++) {
          int dist = Math.abs(e.x/24 - getEnt(i).x/24) + Math.abs(e.y/24 - getEnt(i).y/24);

          if(dist < 2) {
            if(getEnt(i).responsive == true && getEnt(i).faction != e.faction) {
              //System.out.println("DetectTalk() returns:"+i);
              return(i);
            }
          }
        }
      }
      /* Walkabout */
      else if(map_type == 1) {
        for(int i = 0; i < targets.length; i++) {
          int dist = Math.abs(e.x/24 - getEnt(i).x/24) + Math.abs(e.y/24 - getEnt(i).y/24);

          if(e.f/2 == 3 && e.x/24 - getEnt(i).x/24 <= 0) { //up
            dist = 99;
          }
          if(e.f/2 == 0 && e.y/24 - getEnt(i).y/24 <= 0) { //right
            dist = 99;
          }
          if(e.f/2 == 1 && e.x/24 - getEnt(i).x/24 >= 0) { //down
            dist = 99;
          }
          if(e.f/2 == 2 && e.y/24 - getEnt(i).y/24 >= 0) { //left
            dist = 99;
          }

          if(dist < 2) {
            if(getEnt(i).responsive == true && getEnt(i).faction != e.faction) {
              //System.out.println("DetectTalk() returns:"+i);
              return(i);
            }
          }
        }
      }
      return(-1);
    }

    // Gets adjacent Sprites
    public static int[] getAdjacentEnts(boolean offensive) {
      Entity e = Player();
      return(getAdjacentEnts(e,e.range/3,e.range,offensive));
    }

    // Determine range for spells
    public static int[] getAdjacentEnts(Spell s) {
      if(s.offensive == false) {
        return(getAdjacentEnts(Player(),-1,s.range,false));
        //minrange = -1 so that you can target yourself with non-offensive spells.
      }
      else {
        return(getAdjacentEnts(Player(),0,s.range,true));
      }
    }

    /* Basic adjacency function */
    public static int[] getAdjacentEnts(Entity e, int minrange, int maxrange, boolean offensive) {
      int[] targets = new int[numEnts()];
      int count = 0;

      for(int i = 0; i < targets.length; i++) {
        int dist = Math.abs(e.x/24 - getEnt(i).x/24) + Math.abs(e.y/24 - getEnt(i).y/24);

        if(dist <= maxrange && dist>minrange) {
          if(offensive == true && getEnt(i).faction != e.faction) {
            targets[count] = i;
            count++;
          }
          else if(offensive == false &&
                 (getEnt(i).faction == 0 || getEnt(i).faction == e.faction)) {
            targets[count] = i;
            count++;
          }
        }
      }

      int[] selected = new int[count];
      for(int i=0; i < count; i++) {
        selected[i] = targets[i];
      }
      return(selected);
    }

    /* Basic adjacency function from different locations*/
    public static int[] getAdjacentEnts(int x, int y, boolean offensive) {
      int[] targets = new int[numEnts()];
      int count = 0;
      int minrange = Player().range/3;
      int maxrange = Player().range;

      for(int i = 0; i < targets.length; i++) {
        int dist = Math.abs(x - getEnt(i).x/24) + Math.abs(y - getEnt(i).y/24);

        if(dist <= maxrange && dist>minrange) {
          if(offensive == true && getEnt(i).faction != Player().faction) {
            targets[count] = i;
            count++;
          }
          else if(offensive == false &&
                 (getEnt(i).faction == 0 || getEnt(i).faction == Player().faction)) {
            targets[count] = i;
            count++;
          }
        }
      }

      int[] selected = new int[count];
      for(int i=0; i < count; i++) {
        selected[i] = targets[i];
      }

      return(selected);
    }

    /* Sequentially follows all players. Temporary?
     *
     * TODO: Move this to BattleFrame.java where it belongs
     */
    public static String nextPlayer() {

      cur_map.restoreObs();

      if(player >= ents.size()-1) { player = 0; }
      else { player++; }

      cur_map.setValue(Player().x/24,Player().y/24,5,0);

      cur_map.placeMoveObs(Buffer.Player());

      if(Player().ai == true) {
        String aimm[][] = Lib_Util.calcAIMove(Player());
        //for(int i = 1; i < (Player().move*2)+2 ; i++) {

        /* Totally random moving! */
        int i = (int) (Math.random() * (Player().move*2)+2);
        int j = (int) (Math.random() * (Player().move*2)+2);
        int max_score = 0;
        String movestring = "X ";

        /*
        while(movestring.charAt(0) == 'X') {
          i = Math.random() * (Player().move*2)+2;
          j = Math.random() * (Player().move*2)+2;
          movestring = aimm[(int)i][(int)j];
        }*/

        int debug_x = 0;
        int debug_y = 0;

        /*for(int k = 0; k < 100; k++) {
          i = (int) (Math.random() * (Player().move*2)+2);
          j = (int) (Math.random() * (Player().move*2)+2);

          int x = Player().x/24 - Player().move - 1 + i;
          int y = Player().y/24 - Player().move - 1 + j;

          System.out.println("x/y are:"+x+" "+y);

          int cur_score = Lib_Util.calcAIScore(x,y);
          if(cur_score > max_score && aimm[i][j].charAt(0) != 'X') {
            max_score = cur_score;
            movestring = aimm[i][j];
            debug_x = x;
            debug_y = y;
          }
        }*/

        for(i = 0; i < (Player().move*2)+3; i++) {
          for(j = 0; j < (Player().move*2)+3; j++) {

          //i = (int) (Math.random() * (Player().move*2)+2);
          //j = (int) (Math.random() * (Player().move*2)+2);

          int x = Player().x/24 - Player().move - 1 + i;
          int y = Player().y/24 - Player().move - 1 + j;

          int cur_score = Lib_Util.calcAIScore(x,y);

          if(cur_score > max_score && aimm[i][j].charAt(0) != 'X') {
            max_score = cur_score;
            movestring = aimm[i][j];
            debug_x = x;
            debug_y = y;
            //System.out.println("x/y are:"+x+" "+y+". New maxscore is:"+max_score);
          }
          }
        }

        //System.out.println("Entity "+player+" at "+(Player().x/24)+","+(Player().y/24)+" moves to "+debug_x+","+debug_y+".\n He uses movestring:"+movestring);

        return(movestring.substring(1) + " ");
      }




      return(" ");
    }
    public static void firstPlayer() { player = 0; }

    //get player entity
    public static Entity Player() {
      return (Entity) ents.elementAt(player);
    }
    public static Entity lastEnt() {
      return (Entity) ents.elementAt(ents.size()-1);
    }

    //get player index
    public static int playerIndex() { return player; }

    //sets Sprite at given index. Smee & Beta only!
    public static void setEnt(Entity s, int index) {
      /* TODO: clean up old entity layer value */
    	ents.setElementAt(s,index);
      if(index != player) { cur_map.setValue(s.x/24 ,s.y/24 ,5 ,ents.size()); }
    }

	  //sets all Sprites. Useful for loading entire new Spritesets
    /* TODO: set appopriate entity layer */
    public static void setEnts(Vector<Entity> s) {
    	ents = s;
    }

    //adds a single new Sprite. Smee & Beta only!
    public static void addEnt(Entity s) {
    	ents.addElement(s);
      if((ents.size()-1) != player) { cur_map.setValue(s.x/24 ,s.y/24 ,5 ,ents.size()); }

      //System.out.println("Entity #"+(ents.size()-1)+" has value "+cur_map.getValue(s.x/24,s.y/24,5));
      /* WARNING: Watch out for this mapping!
       * Entity 0 is mapped to value 1 on the map entity layer etc!
       */
    }

    //deletes a single Sprite. Smee & Beta only!
    public static void delEnt(int index) {
    	ents.removeElementAt(index);
    }

	/* Note: Smee camera stuff is done within SmeePanel,
	 * because it's far less complicated than the game camera system
	 */



	/* SMEE Stuff */

	/* Graphic for obstructed tile! */
	static private Image obs, zone;
	public static void setObsImage(Image i) {
		obs = i;
	}
  public static void setZoneImage(Image i) {
    zone = i;
  }
	public static Image getObsImage() {
		return obs;
	}
  public static Image getZoneImage() {
    return zone;
  }

  static private Image[] flash = new Image[2];
  public static void setFlashImage(Image i, Image j) {
    flash[0] = i;
    flash[1] = j;
  }
  public static Image getFlashImage() {
    return flash[anim_toggle];
  }

  static private Image[] flash2 = new Image[2];
  public static void setAtkRangeImage(Image i, Image j) {
    flash2[0] = i;
    flash2[1] = j;
  }
  public static Image getAtkRangeImage() {
    return flash2[anim_toggle];
  }

	private static int cur_layer; // selected layer
	private static int curTile; // selected tile
	private static Color curLColor = new Color(0,0,0); // selected color
  private static Color curRColor = new Color(0,0,0); // selected color

	public static void setCurColor(Color a, int lr)
	{
    if(lr == 0) { curLColor = a; }
    else { curRColor = a;  }
	}

	public static Color getCurColor(int lr)
	{
    if(lr == 0) { return(curLColor); }
    else { return(curRColor); }
  }

	public static void setCurTile(int a)
	{	curTile = a; }

	public static int getCurTile()
	{  	return curTile;	}

	public static void setCurLayer(int value)
	{	Buffer.cur_layer = value; }

	public static int getCurLayer()
	{	return Buffer.cur_layer; }



	/*
	 *TILE-BUFFERING SECTION
	 *Necessary for both game and Smee!
	 */

	private static Vector<BufferedImage> tiles;

	public static void initializeTiles(WalkMap cur_map)
	/*Retrieves and save the BufferedImage objects from the tiles  of the
	 *current map.
	 */
	{
		int length = cur_map.getTileset().getLength();

		tiles = new Vector<BufferedImage>();

		for(int i=0; i<length; i++)
		{
			tiles.addElement(cur_map.getTileset().getTile(i).getSource());
		}
	}

	public static BufferedImage getTile(int index)
	/*Returns a specific tile image
	 */
	{
		return (BufferedImage) tiles.elementAt(index);
	}

	public static void addTile()
	{
		tiles.addElement(new Tile().getSource());
	}

 public static int findItem(String name)
 {
  for(int i=0; i<items.size(); i++)
  {
   if(items.elementAt(i).wcallname.equals(name) == true)
   {
    return i;
   }
  }
  return(1);
 }

 public static Items getItem(int index)
 {
  return (Items) items.elementAt(index);
 }

 public static int findShop(String name, String type)
 {
  for(int i=0; i<shop.size(); i++)
  {
   if(shop.elementAt(i).shopformap.equals(name) == true && shop.elementAt(i).shoptype.equals(type) == true)
   {
    return i;
   }
  }
  return(-1);
 }

 public static Shops getShop(int index)
 {
  return (Shops) shop.elementAt(index);
 }
}
