import java.awt.*; import javax.swing.*; import java.net.URL; import java.awt.image.*;
import java.io.*; import javax.imageio.*;

/*
	Lib_Image.java
	(c)1999-2008 SSC Development Team
	Function: Managing basic image manipulation and retrieval functions.
	Allowed Dependencies: None
*/

public class Lib_Image
{
	public static Image getImage(String filename)
	{
  /*try {
    return ImageIO.read(new File(filename));
  }
  catch (IOException e) {
  }
  return(null);*/
  return new ImageIcon(filename).getImage();
	//return Toolkit.getDefaultToolkit().createImage(file name);
	}

	public static Image getImage(ImageProducer fis)
	{
	return Toolkit.getDefaultToolkit().createImage(fis);
	}

	public static BufferedImage cropImage(Image source, int x, int y, int w, int h)
	{
    //I commented out the deprecated version. Code might still be handy for other stuff though
    //ImageFilter         filter = new CropImageFilter(x, y, w, h);
    //FilteredImageSource fis    = new FilteredImageSource(source.getSource(),filter);
    //return getImage(fis);
    BufferedImage source2 = toBufferedImage(source);
    return(source2.getSubimage(x,y,w,h));
    }


	// Converts Image to BufferedImage
    public static BufferedImage toBufferedImage(Image image) {
        if (image instanceof BufferedImage) { return (BufferedImage)image; }

        // This code ensures that all the pixels in the image are loaded
        image = new ImageIcon(image).getImage();

        // Determine if the image has transparent pixels; for this method's
        // implementation, see e665 Determining If an Image Has Transparent Pixels
        boolean hasAlpha = hasAlpha(image);

        // Create a buffered image with a format that's compatible with the screen
        BufferedImage bimage = null;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        try {
            // Determine the type of transparency of the new buffered image
            int transparency = Transparency.OPAQUE;
            if (hasAlpha) {
                transparency = Transparency.BITMASK;
            }

            // Create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            bimage = gc.createCompatibleImage(
                image.getWidth(null), image.getHeight(null), transparency);
        } catch (HeadlessException e) {
            // The system does not have a screen
        }

        if (bimage == null) {
            // Create a buffered image using the default color model
            int type = BufferedImage.TYPE_INT_RGB;
            if (hasAlpha) {
                type = BufferedImage.TYPE_INT_ARGB;
            }
            bimage = new BufferedImage(image.getWidth(null), image.getHeight(null), type);
        }

        // Copy image to buffered image
        Graphics g = bimage.createGraphics();

        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();

        return bimage;
    }

    // This method returns true if the specified image has transparent pixels
    private static boolean hasAlpha(Image image) {
        // If buffered image, the color model is readily available
        if (image instanceof BufferedImage) {
            BufferedImage bimage = (BufferedImage)image;
            return bimage.getColorModel().hasAlpha();
        }

        // Use a pixel grabber to retrieve the image's color model;
        // grabbing a single pixel is usually sufficient
         PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
        try {
            pg.grabPixels();
        } catch (InterruptedException e) {
        }

        // Get the image's color model
        ColorModel cm = pg.getColorModel();
        return cm.hasAlpha();
    }

   public static VolatileImage fastDraw(Graphics2D g, VolatileImage img,
                                           int x, int y, Image orig) {
        final int MAX_TRIES = 100;
        for (int i=0; i<MAX_TRIES; i++) {
            if (img != null) {
                // Draw the volatile image
                g.drawImage(img, x, y, null);

                // Check if it is still valid
                if (!img.contentsLost()) {
                    return img;
                }
            } else {
                // Create the volatile image
                img = g.getDeviceConfiguration().createCompatibleVolatileImage(
                    orig.getWidth(null), orig.getHeight(null));
            }

            // Determine how to fix the volatile image
            switch (img.validate(g.getDeviceConfiguration())) {
            case VolatileImage.IMAGE_OK:
                // This should not happen
                break;
            case VolatileImage.IMAGE_INCOMPATIBLE:
                // Create a new volatile image object;
                // this could happen if the component was moved to another device
                img.flush();
                img = g.getDeviceConfiguration().createCompatibleVolatileImage(
                    orig.getWidth(null), orig.getHeight(null));
            case VolatileImage.IMAGE_RESTORED:
                // Copy the original image to accelerated image memory
                Graphics2D gc = (Graphics2D)img.createGraphics();
                gc.drawImage(orig, 0, 0, null);
                gc.dispose();
                break;
            }
        }

        // The image failed to be drawn after MAX_TRIES;
        // draw with the non-accelerated image
        g.drawImage(orig, x, y, null);
        return img;
    }

    public static VolatileImage fastADraw(Graphics2D g, VolatileImage img,
                                           int x, int y, float alpha, Image orig) {
        final int MAX_TRIES = 100;
        for (int i=0; i<MAX_TRIES; i++) {
            if (img != null) {
                // Draw the volatile image

                Graphics2D g2d = (Graphics2D) g;

                g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));

                g.drawImage(img, x, y, null);

                g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));

                // Check if it is still valid
                if (!img.contentsLost()) {
                    return img;
                }
            } else {
                // Create the volatile image
                img = g.getDeviceConfiguration().createCompatibleVolatileImage(
                    orig.getWidth(null), orig.getHeight(null));
            }

            // Determine how to fix the volatile image
            switch (img.validate(g.getDeviceConfiguration())) {
            case VolatileImage.IMAGE_OK:
                // This should not happen
                break;
            case VolatileImage.IMAGE_INCOMPATIBLE:
                // Create a new volatile image object;
                // this could happen if the component was moved to another device
                img.flush();
                img = g.getDeviceConfiguration().createCompatibleVolatileImage(
                    orig.getWidth(null), orig.getHeight(null));
            case VolatileImage.IMAGE_RESTORED:
                // Copy the original image to accelerated image memory
                Graphics2D gc = (Graphics2D)img.createGraphics();
                gc.drawImage(orig, 0, 0, null);
                gc.dispose();
                break;
            }
        }

        // The image failed to be drawn after MAX_TRIES;
        // draw with the non-accelerated image
        g.drawImage(orig, x, y, null);
        return img;
    }

  	/* Creates an ImageIcon. */
    protected static ImageIcon getIcon(String path) {
	    return new ImageIcon(path);
    }

  /* Generic box drawing function */
  public static void drawWindow(Graphics g, int x, int y, int h, int w)
  {
    g.setColor(new Color(0,36,128));//blue fill
    g.fillRoundRect(x,y,h,w, 4, 4);
    g.setColor(Color.black);//black outerborder
    g.drawRoundRect(x,y,h,w, 4, 4);
    g.setColor(new Color(144,72,16));//brown
    g.drawRoundRect(x+1,y+1,h-2,w-2, 4, 4);
    g.setColor(new Color(248,216,88));//yellow
    g.drawRoundRect(x+2,y+2,h-4,w-4, 4, 4);
    g.setColor(Color.white);//white
    g.drawRoundRect(x+3,y+3,h-6,w-6, 4, 4);
    g.setColor(new Color(144,72,16)); //brown again
    g.drawRoundRect(x+4,y+4,h-8,w-8, 4, 4);
  }

  public static void textBox(Graphics g, String a, String b, String c)
  {
    g.setFont(Buffer.italictxt);
    drawWindow(g,4,170,351,65);
    g.setColor(new Color(255,255,192));
    g.drawString(a,10,190);
    g.drawString(b,10,205);
    g.drawString(c,10,220);
  }


  /* Self splitting string TextBox */
  public static void textBox(Graphics g, String source)
  {

    //source = Buffer.getEnt(Buffer.DetectTalk()).resptext;

    if(source == null) {
      System.out.println("Error: empty String in textBox!");
    }
    else {

      String a,b,c;
      a = ""; b = ""; c = "";

      String temp = "";
      int i = 0;
      int j = 0;
      source += " \\s";
      String chunks[] = source.split("\\s");
      while (i<chunks.length) {
        if(temp.length() + chunks[i].length() < 60 ) {
          temp += " "+chunks[i];

        if (i+1<chunks.length) {
            if(j == 0) { a = temp; }
            else if(j == 1) { b = temp; }
            else if(j == 2) { c = temp; }
          }
        }
        else {
          if(j == 0) { a = temp; temp = chunks[i]; j++; }
          else if(j == 1) { b = temp; temp = chunks[i]; j++; }
          else if(j == 2) { c = temp; System.out.println("Warning: String is too long!"); }
        }
        i++;
      }
      Lib_Image.textBox(g,a,b,c);
    }

  }



  public static void battleBox(Graphics g, String a, String b, String c)
  {
    g.setFont(Buffer.normaltxt);
    drawWindow(g,96,190,259,45);
    g.setColor(new Color(255,255,192));
    g.drawString(a,102,205);
    g.drawString(b,102,215);
    g.drawString(c,102,225);
  }

  public static void spellMenu(Graphics g, Entity e, int choice, int levelchoice)
  {
    g.setFont(Buffer.italictxt);
    drawWindow(g,86,90,140,90);

    //level boxes
    g.setColor(new Color(96,255,96));
    for(int h=0;h<4;h++) {
      if(e.spell[h] != null) {
        int limit;

        limit = e.spell[h].level;

        for (int i=0;i<limit;i++) {
          g.fillRect(172+(i*11),98+(h*15),10,5);
        }

        if(h == choice) {
          g.setColor(new Color(255,96,96));
          for (int i=levelchoice;i<limit;i++) {
            g.fillRect(172+(i*11),98+(h*15),10,5);
          }
          g.setColor(new Color(96,255,96));
        }
      }
    }

    g.setColor(new Color(255,255,192));
    g.drawString(e.spell[0].shortname,102,105);
    if(e.spell[1] != null) {
      g.drawString(e.spell[1].shortname,102,120);
    }
    if(e.spell[2] != null) {
      g.drawString(e.spell[2].shortname,102,135);
    }
    if(e.spell[3] != null) {
      g.drawString(e.spell[3].shortname,102,150);
    }
    g.fillRect(92,98+(choice*15),5,5);
  }

  /* TODO: Move these to a more appopriate location
   * Lib_Interface perhaps??
   */
  public static void playerStatBox(Graphics g)
  {      //this draws the HP MP box in top right corner
    g.setFont(Buffer.smalltxt);

    int mbl = Math.max(20,Math.min(50,Math.max(Buffer.Player().maxhp,Buffer.Player().maxhp)/2));
    mbl=mbl+30;    //new change (10/01/06) Elvenfyre
    int offset = 327 - mbl;

    g.setColor(new Color(0,0,192));
    drawWindow(g,offset-19,3,47+mbl,33);

    g.setColor(new Color(255,255,192));
    g.drawString(Buffer.Player().name+" lv:"+Buffer.Player().level,offset-14,14);

    //draw hp
    g.drawString("hp:",offset-14,22);
    String t1 = ""+Buffer.Player().curhp;
    if(Buffer.Player().curhp > 99) {
      t1 = "??";
    }
    String t2 = ""+Buffer.Player().maxhp;
    if(Buffer.Player().maxhp > 99) {
      t2 = "??";
    }
    g.drawString(t1+"/"+t2,330,22);

    //draw mp
    g.drawString("mp:",offset-14,31);
    t1 = ""+Buffer.Player().curmp;
    if(Buffer.Player().curmp > 99) {
      t1 = "??";
    }
    t2 = ""+Buffer.Player().maxmp;
    if(Buffer.Player().maxmp > 99) {
      t2 = "??";
    }
    g.drawString(t1+"/"+t2,330,31);

    int len = Math.min(Buffer.Player().curhp/2,50);
    g.setColor(new Color(192,0,0));
    g.fillRect(offset,18,Math.min(Buffer.Player().maxhp/2,50),4);
    g.setColor(new Color(255,255,128));
    g.fillRect(offset,18,len,4);

    int maxmp = Buffer.Player().maxmp;
    if(maxmp > 0) {
      len = (Math.min(Buffer.Player().curmp/2,50));

      g.setColor(new Color(192,0,0));
      g.fillRect(offset,27,Math.min(Buffer.Player().maxmp/2,50),4);
      g.setColor(new Color(255,255,128));
      g.fillRect(offset,27,len,4);

      g.setColor(new Color(64,64,0));
      g.drawRect(offset-1,26,(Math.min((Buffer.Player().maxmp/2)+2,52)),6);
    }
    g.setColor(new Color(64,64,0));
    g.drawRect(offset-1,17,Math.min((Buffer.Player().maxhp/2)+2,52),6);
  }

  public static void targetStatBox(Graphics g, Entity e)
  {
    int mbl = Math.max(20,Math.min(50,Math.max(e.maxhp,e.maxhp)/2));
    mbl=mbl+30;    //new change (10/01/06) Elvenfyre
    g.setFont(Buffer.smalltxt);

    g.setColor(new Color(0,0,192));
    drawWindow(g,3,200,47+mbl,33);

    g.setColor(new Color(255,255,192));
    g.drawString(e.name+" lv:"+e.level,8,211);

    g.drawString("hp:",8,219);
    String t1 = ""+e.curhp;
    if(e.curhp > 99) {
      t1 = "??";
    }
    String t2 = ""+e.maxhp;
    if(e.maxhp > 99) {
      t2 = "??";
    }
    g.drawString(t1+"/"+t2,24+mbl,219);

    g.drawString("mp:",8,228);
    t1 = ""+e.curmp;
    if(e.curmp > 99) {
      t1 = "??";
    }
    t2 = ""+e.maxmp;
    if(e.maxmp > 99) {
      t2 = "??";
    }
    g.drawString(t1+"/"+t2,24+mbl,228);

    int len = Math.min(e.curhp/2,50);
    g.setColor(new Color(192,0,0));
    g.fillRect(22,215,Math.min(e.maxhp/2,50),4);
    g.setColor(new Color(255,255,128));
    g.fillRect(22,215,len,4);

    int maxmp = e.maxmp;
    if(maxmp > 0) {
      len = Math.min(e.curmp/2,50);

      g.setColor(new Color(192,0,0));
      g.fillRect(22,224,Math.min(e.maxmp/2,50),4);
      g.setColor(new Color(255,255,128));
      g.fillRect(22,224,len,4);

      g.setColor(new Color(64,64,0));
      g.drawRect(21,223,(Math.min(e.maxmp/2,50)+2),6);
    }
    g.setColor(new Color(64,64,0));
    g.drawRect(21,214,Math.min(e.maxhp/2,50)+2,6);
  }

  public static void statsPage(Graphics g)
  {
    g.setFont(Buffer.normaltxt);

    g.setColor(new Color(0,0,192));
    drawWindow(g,10,10,90,85); //portrait box
    drawWindow(g,10,95,90,100); //sprite and kills/deaths box
    drawWindow(g,10,195,90,35); //gold box
    drawWindow(g,100,10,250,220);  //main stats box

    g.setColor(new Color(255,255,192));
    g.drawString(Buffer.Player().name+" lv:"+Buffer.Player().level,110,30);

    int len = (100 * Buffer.Player().curhp) / Buffer.Player().maxhp;
    g.drawString("HP:", 115, 50);
    g.setColor(new Color(192,0,0));
    g.fill3DRect(135,42,100,10,true);
    g.setColor(new Color(255,255,192));
    g.fill3DRect(135,42,len,10,true);
    g.drawString(Buffer.Player().curhp+"/"+Buffer.Player().maxhp,240,50);

    int maxmp = Buffer.Player().maxmp;
    g.drawString("MP:", 115, 65);
    if(maxmp > 0) {
      len = (100 * Buffer.Player().curmp) / maxmp;
      g.setColor(new Color(192,0,0));
      g.fill3DRect(135,57,100,10,true);
      g.setColor(new Color(255,255,192));
      g.fill3DRect(135,57,len,10,true);
      g.drawString(Buffer.Player().curmp+"/"+Buffer.Player().maxmp,240,65);
    }
    else
    {//if their mp is 0/0
     g.drawString(Buffer.Player().curmp+"/"+Buffer.Player().maxmp,130,70);
    }
    //draw portrait
    g.drawImage(Buffer.Player().portrait,15,15,null);

    g.setColor(new Color(255,255,128));
    g.drawString("Strength",115,85);
    g.drawString("Precision",115,97);
    g.drawString("Reflexes",115,109);
    g.drawString("Endurance",115,121);
    g.drawString("Speed",115,133);
    g.drawString("Willpower",115,145);

    g.setColor(new Color(128,255,145));
    g.drawString("Atk chance",210,85);
    g.drawString("Dge chance",210,97);

    g.setColor(new Color(255,128,128));
    g.drawString("Damage",210,109);

    g.setColor(new Color(196,196,196));
    g.drawString("Armor",210,121);
    g.drawString("Movement",210,133);
    g.drawString("Spell Res.",210,145);

    Entity e = Buffer.Player();

    g.setColor(new Color(255,255,128));
    g.drawString(e.strength+"",170,85);
    g.drawString(e.precision+"",170,97);
    g.drawString(e.reflexes+"",170,109);
    g.drawString(e.endurance+"",170,121);
    g.drawString(e.speed+"",170,133);
    g.drawString(e.willpower+"",170,145);

    g.setColor(new Color(128,255,145));
    g.drawString(e.att+"%",270,85);
    g.drawString(e.dod+"%",270,97);

    g.setColor(new Color(255,128,128));
    g.drawString(e.mindam+"-"+(e.mindam+e.damran),270,109);

    g.setColor(new Color(196,196,185));
    g.drawString(e.armor+"",270,121);
    g.drawString(e.move+"",270,133);
    g.drawString(e.spellres+"",270,145);

    //do magic
    g.setColor(new Color(255,255,192));
    g.drawString("Talents/Magic:",110,170);
    if(e.spell[0]!=null)
    {
     g.drawString(e.spell[0].name, 115,182);
    }
    if(e.spell[1]!=null)
    {
    g.drawString(e.spell[1].name, 115,194);
    }
    if(e.spell[2]!=null)
    {
    g.drawString(e.spell[2].name, 115,206);
    }
    if(e.spell[3]!=null)
    {
    g.drawString(e.spell[3].name, 115,218);
    }


    //do items
    // no items yet.


  }

  public static void terrainStatBox(Graphics g)
  {
    g.setFont(Buffer.smalltxt);

    g.setColor(new Color(0,0,192));
    drawWindow(g,3,4,82,30);

    g.setColor(new Color(255,255,192));
    g.drawString("Land Type: ",6,14);
    String temp = Stats.getTileLE(Buffer.Player().x/24,Buffer.Player().y/24);
    int temp2;
    if(Buffer.Player().range<3) {
      temp2 = Stats.getTileMeleeMiss(Buffer.Player().x/24,Buffer.Player().y/24);
    }
    else {
      temp2 = Stats.getTileMissileMiss(Buffer.Player().x/24,Buffer.Player().y/24);
    }

    g.drawString(temp+"  ("+temp2+"%)",6,26);

  }

  //TODO: Implement!
  //public static void renderSprites(Entities ent, Graphics g, JPanel m, int x, int y, int w, int h)

  private static VolatileImage[] tile_v;
  //private static VolatileImage[] flash_tile_v;

	public static void initMap(WalkMap map) {
	  tile_v = new VolatileImage[map.getTileset().getLength()];
	  //flash_tile_v = new VolatileImage[2];
	}

	public static void renderMap(WalkMap map, Graphics g, JPanel m, int x, int y, int w, int h, int layer, int vol)
	{
		int ini_x = x;
		int ini_y = y;

		int width = map.getWidth();
		int height = map.getHeight();

		if(ini_x<0) ini_x=0;
		if(ini_y<0) ini_y=0;

		int max_x = x+w;
		int max_y = y+h;

		int max_tx = (ini_x+w)/24;
		int max_ty = (ini_y+h)/24;

		if(max_tx >= width) {
			max_tx = width-1;
			max_x = width*24-1;
			ini_x = max_x-w;
			if(ini_x<0) {
				ini_x = 0;
				}
			}
		if(max_ty >= height) {
			max_ty = height-1;
			max_y = height*24-1;
			ini_y = max_y-h;
			if(ini_y<0) {
				ini_y = 0;
				}
			}

		if(layer==0 && vol == 1) {
			for(int i=ini_x/24;i<=max_tx;i++) {
			for(int j=ini_y/24;j<=max_ty;j++) {
          int p = map.getValue(i,j,0);

          tile_v[p] = Lib_Image.fastDraw((Graphics2D) g, tile_v[p], 24*i-ini_x, 24*j-ini_y, map.getBlock(i,j,0));
					//g.drawImage(map.getBlock(i,j,0), 24*i-ini_x, 24*j-ini_y, m);
				}
			}
		}
		else if(layer==0 && vol == 0) {
      for(int i=ini_x/24;i<=max_tx;i++) {
      for(int j=ini_y/24;j<=max_ty;j++) {

          //tile_v[i][j] = Lib_Image.fastDraw((Graphics2D) g, tile_v[i][j], 24*i-ini_x, 24*j-ini_y, map.getBlock(i,j,0));
          g.drawImage(map.getBlock(i,j,0), 24*i-ini_x, 24*j-ini_y, m);
        }
      }
		}

		if(layer==1) {
			for(int i=ini_x/24;i<=max_tx;i++) {
			for(int j=ini_y/24;j<=max_ty;j++) {
					if(map.getValue(i,j,1)>0) {
          //tile_v[i][j][1] = Lib_Image.fastDraw((Graphics2D) g, tile_v[i][j][1], 24*i-ini_x, 24*j-ini_y, map.getBlock(i,j,1));
					g.drawImage(map.getBlock(i,j,1), 24*i-ini_x, 24*j-ini_y, m);
					//slow drawing because VolatileImages don't seem to support transparency
					}
				}
			}
		}

		if(layer==2) {
			for(int i=ini_x/24;i<=max_tx;i++) {
			for(int j=ini_y/24;j<=max_ty;j++) {
					if(Buffer.visibleRoofs == true) {
					  if(map.getValue(i,j,2)>0) {
              //tile_v[i][j][2] = Lib_Image.fastDraw((Graphics2D) g, tile_v[i][j][2], 24*i-ini_x, 24*j-ini_y, map.getBlock(i,j,2));
					    g.drawImage(map.getBlock(i,j,2), 24*i-ini_x, 24*j-ini_y, m);
              //slow drawing because VolatileImages don't seem to support transparency
            }
					}
					else {
            if(map.getValue(i,j,2)>0 && map.detectRoof(i,j) == false) {
              g.drawImage(map.getBlock(i,j,2), 24*i-ini_x, 24*j-ini_y, m);
            }
          }
				}
			}
		}

		if(layer==3) {
			for(int i=ini_x/24;i<=max_tx;i++) {
			for(int j=ini_y/24;j<=max_ty;j++) {
					if(map.getValue(i,j,3) == 1) {
						g.drawImage(Buffer.getObsImage(), 24*i-ini_x, 24*j-ini_y, m);
					}
				}
			}
		}

    if(layer==4) {
      for(int i=ini_x/24;i<=max_tx;i++) {
      for(int j=ini_y/24;j<=max_ty;j++) {
          if(map.getValue(i,j,4) > 0) {
            g.drawImage(Buffer.getZoneImage(), 24*i-ini_x, 24*j-ini_y, m);
          }
        }
      }
    }


    /* Fourth Layer */
    if(layer==6) {
      for(int i=ini_x/24;i<=max_tx;i++) {
      for(int j=ini_y/24;j<=max_ty;j++) {
          if(Buffer.visibleRoofs == true) {
            if(map.getValue(i,j,6)>0) {
            //tile_v[i][j][1] = Lib_Image.fastDraw((Graphics2D) g, tile_v[i][j][1], 24*i-ini_x, 24*j-ini_y, map.getBlock(i,j,1));
            g.drawImage(map.getBlock(i,j,6), 24*i-ini_x, 24*j-ini_y, m);
            //slow drawing because VolatileImages don't seem to support transparency
            }
          }
        }
      }
    }

		if(layer==100) { //Flashing tiles!
      int[][] m_mat = WalkMap.move_matrix;
      int rsx = Buffer.MMorigX;
      int rsy = Buffer.MMorigY;

      int mm_len = (Buffer.Player().move*2)+2;

      for(int x2=1; x2<mm_len; x2++)
      {
        for(int y2=1; y2<mm_len; y2++)
        {

          if(Buffer.getCurMap().getValue(rsx+x2,rsy+y2,3) != 1 &&
             m_mat[x2][y2] > 0 && rsx+x2>=0 && rsy+y2>=0 &&
             rsx+x2<map.getWidth() && rsy+y2<map.getHeight()) {

            //flash_tile_v[Buffer.anim_toggle] = Lib_Image.fastADraw((Graphics2D) g, flash_tile_v[Buffer.anim_toggle], (24*(rsx+x2)-ini_x), (24*(rsy+y2)-ini_y), Buffer.getFlashImage());

            g.drawImage(Buffer.getFlashImage(), (24*(rsx+x2)-ini_x), (24*(rsy+y2)-ini_y), m);
          }
        }
      }
   	}

	}



}
