import java.util.*; import java.awt.*; import java.io.*; import java.awt.image.*;
import javax.swing.*;

/*
	Lib_Util.java
	(c)1999-2008 SSC Development Team
	Function: Managing generally useful functions.
	Allowed Dependencies: None
*/

public class Lib_Util
{


/* Calculate obstacles to be placed for player move range
 * using Devlyn's ugly Bucketfill algorithm ;)
 */

public static int[][] calcMoveArea(Entity e)
{
  int matrix[][] = new int[(e.move*2)+3][(e.move*2)+3];
  int mp[][] = new int[(e.move*2)+3][(e.move*2)+3];
  int width = (e.move*2)+2;
  int movepoints = e.move*10;

  matrix[e.move+1][e.move+1] = 1;

  for(int i=1; i<width; i++) {
    for(int x=1; x<width; x++)
    {
      for(int y=1; y<width; y++)
      {
        if(matrix[x][y] == i) {

          int rx = (e.x/24) - e.move - 1 + x;
          int ry = (e.y/24) - e.move - 1 + y;

          //System.out.println("x/y:"+rx+" "+ry+" mp:"+mp[x][y]);

          if(matrix[x-1][y] == 0 && rx>0) {
             mp[x-1][y] = mp[x][y] + Stats.getTileMoveCost(rx-1, ry);
             if(mp[x-1][y] > movepoints) {
               matrix[x-1][y] = -1;
             }
             else {
               matrix[x-1][y] = i+1;
             }
          }

          if(matrix[x+1][y] == 0 && rx<Buffer.getCurMap().getHeight()-1) {
             mp[x+1][y] = mp[x][y] + Stats.getTileMoveCost(rx+1, ry);
             if(mp[x+1][y] > movepoints) {
               matrix[x+1][y] = -1;
             }
             else {
               matrix[x+1][y] = i+1;
             }
          }

          if(matrix[x][y-1] == 0 && ry>0) {
             mp[x][y-1] = mp[x][y] + Stats.getTileMoveCost(rx, ry-1);
             if(mp[x][y-1] > movepoints) {
               matrix[x][y-1] = -1;
             }
             else {
               matrix[x][y-1] = i+1;
             }
          }

          if(matrix[x][y+1] == 0 && ry<Buffer.getCurMap().getWidth()-1) {
             mp[x][y+1] = mp[x][y] + Stats.getTileMoveCost(rx, ry+1);
             if(mp[x][y+1] > movepoints) {
               matrix[x][y+1] = -1;
             }
             else {
               matrix[x][y+1] = i+1;
             }
          }
        }
      }
    }
  }

return(matrix);

}


/* AI scoring algorithm */
public static String[][] calcAIMove(Entity e)
{
  int matrix[][] = new int[(e.move*2)+3][(e.move*2)+3];
  //movescripts = ms
  String ms[][] = new String[(e.move*2)+3][(e.move*2)+3];
  int mp[][] = new int[(e.move*2)+3][(e.move*2)+3];
  int width = (e.move*2)+2;
  int movepoints = e.move*10;

  matrix[e.move+1][e.move+1] = 1;

  for(int x=0; x<(e.move*2)+3; x++)
  {
    for(int y=0; y<(e.move*2)+3; y++)
    {
      ms[x][y] = "X ";
    }
  }

  for(int i=1; i<width; i++) {
    for(int x=1; x<width; x++)
    {
      for(int y=1; y<width; y++)
      {
        if(matrix[x][y] == i) {

          if(i == 1) { ms[x][y] = " "; }

          int rx = (e.x/24) - e.move - 1 + x;
          int ry = (e.y/24) - e.move - 1 + y;

          //System.out.println("x/y:"+rx+" "+ry+" mp:"+mp[x][y]);

          int ent_val = Buffer.getCurMap().getValue(rx-1,ry,5);
          if(matrix[x-1][y] == 0 && rx>0) {
             mp[x-1][y] = mp[x][y] + Stats.getTileMoveCost(rx-1, ry);
             if(mp[x-1][y] > movepoints || (ent_val > 0 && Buffer.getEnt(ent_val-1).faction != e.faction)) {
               matrix[x-1][y] = -1;
               ms[x-1][y] = "X ";
             }
             else {
               matrix[x-1][y] = i+1;
               ms[x-1][y] = ms[x][y] + "l";
             }
          }

          ent_val = Buffer.getCurMap().getValue(rx+1,ry,5);
          if(matrix[x+1][y] == 0 && rx<Buffer.getCurMap().getHeight()-1) {
             mp[x+1][y] = mp[x][y] + Stats.getTileMoveCost(rx+1, ry);
             if(mp[x+1][y] > movepoints || (ent_val > 0 && Buffer.getEnt(ent_val-1).faction != e.faction)) {
               matrix[x+1][y] = -1;
               ms[x+1][y] = "X ";
             }
             else {
               matrix[x+1][y] = i+1;
               ms[x+1][y] = ms[x][y] + "r";
             }
          }

          ent_val = Buffer.getCurMap().getValue(rx,ry-1,5);
          if(matrix[x][y-1] == 0 && ry>0) {
             mp[x][y-1] = mp[x][y] + Stats.getTileMoveCost(rx, ry-1);
             if(mp[x][y-1] > movepoints || (ent_val > 0 && Buffer.getEnt(ent_val-1).faction != e.faction)) {
               matrix[x][y-1] = -1;
               ms[x][y-1] = "X ";
             }
             else {
               matrix[x][y-1] = i+1;
               ms[x][y-1] = ms[x][y] + "u";
             }
          }

          ent_val = Buffer.getCurMap().getValue(rx,ry+1,5);
          if(matrix[x][y+1] == 0 && ry<Buffer.getCurMap().getWidth()-1) {
             mp[x][y+1] = mp[x][y] + Stats.getTileMoveCost(rx, ry+1);
             if(mp[x][y+1] > movepoints || (ent_val > 0 && Buffer.getEnt(ent_val-1).faction != e.faction)) {
               matrix[x][y+1] = -1;
               ms[x][y+1] = "X ";
             }
             else {
               matrix[x][y+1] = i+1;
               ms[x][y+1] = ms[x][y] + "d";
             }
          }
        }
      }
    }
  }

  /* Post processing: make sure the entity doesn't end on
   * a square occupied by other entities.
   *
   */
  for(int x=0; x<(e.move*2)+3; x++)
  {
    for(int y=0; y<(e.move*2)+3; y++)
    {
      int rx = (e.x/24) - e.move - 1 + x;
      int ry = (e.y/24) - e.move - 1 + y;

      if(Buffer.getCurMap().getValue(rx,ry,5) > 0) {
        ms[x][y] = "X ";
        //System.out.println(rx+" "+ry+": blocked");
      }
    }
  }

return(ms);

}

public static int calcAIScore(int x, int y) {
  int score = 100 - (Math.abs(Buffer.StartX-x) + Math.abs(Buffer.StartY-y));
  int[] adj_ents = Buffer.getAdjacentEnts(x,y,true);

  if(adj_ents.length > 0) {
    score += 25;
  }

  return score;
}

public static Tileset ImportIntoTileset(String filename, Tileset b) {

	//TODO:Check if file exists and if it's the right kind of image!
	Tileset imported = new Tileset(filename);
	for(int i=0; i<imported.getLength(); i++) {
		b.addTile();
		b.setTile(b.getLength()-1,imported.getTile(i));
		}

	return b;
	}

public static void configInit()
{
  Properties config = Lib_Util.ReadProperties("config/settings.cfg");

  int fd = Integer.parseInt(config.getProperty("FullDraw","0"));
  if(fd > 0) { Buffer.FullDraw = true; }
  else { Buffer.FullDraw = false; }

  int mo = Integer.parseInt(config.getProperty("MusicOn","0"));
  if(mo > 0) { Buffer.MusicOn = true; }
  else { Buffer.MusicOn = false; }

  int aae = Integer.parseInt(config.getProperty("AnimateAllEnts","0"));
  if(aae > 0) { Buffer.AnimateAllEnts = true; }
  else { Buffer.AnimateAllEnts = false; }

  Buffer.TimerDelay = Integer.parseInt(config.getProperty("TimerDelay","13"));
  Buffer.AnimDelay = Integer.parseInt(config.getProperty("AnimDelay","20"));
  Buffer.PixelSkip = Integer.parseInt(config.getProperty("PixelSkip","2"));
  Buffer.ScaleRatio = Double.parseDouble(config.getProperty("ScaleRatio","2.0"));
  Buffer.Intro = Integer.parseInt(config.getProperty("Intro","1"));
}

/* Loads a map along with all sprites and tiles: Game only!
 * returns 0 if battle map, 1 if walkabout map
 */
public static int loadMap(String name)
{
  //initPartyFile(); //temp!
  int type = initScriptFile("maps/"+name+".scriptfile");
  Buffer.map_type = type;
  //Added by elvenfyre to enable placement of entityfile name in scriptfile

  if(Buffer.map_type!=2){
  if(Buffer.entityFile==null){
	Buffer.entityFile=name;
  }
  if(Buffer.mapFile==null){
	  	Buffer.mapFile=name;
	}
  //end elvenfyres addition

  try {
    // Deserialize from a file
    File file = new File("maps/"+Buffer.mapFile+".smf"); //starting map is start.smf!
    ObjectInputStream in = new ObjectInputStream(new FileInputStream(file));
    // Deserialize the object
    Object o = in.readObject();
    Buffer.setCurMap(Lib_Util.SMF_WalkMap12WalkMap((SMF_WalkMap1) o));
    Buffer.initializeTiles(Buffer.getCurMap());
    in.close();
      }
  catch (ClassNotFoundException e)
     {
     System.out.println(e);
     System.out.println("maps/"+Buffer.mapFile+".smf seems to be corrupted or in an unrecognized format!");
     }
  catch (IOException e)
     {
     System.out.println(e);
     System.out.println("maps/"+Buffer.mapFile+".smf refuses to load!");
     }

  //must be done after both entity file has been read as well as map tileset painted on
  initEntityFile("maps/"+Buffer.entityFile+".entityfile", type);
  placeParty();

  //for(int i = 0; i < Buffer.numEnts(); i++) {
  //  System.out.println("I:"+i+" resptext = "+Buffer.getEnt(i).resptext);
  //}

  // Load tiles as volatile images!

   Lib_Image.initMap(Buffer.getCurMap());
  } else {
	  //movie
	  if(Buffer.frameFile==null){
		  Buffer.frameFile=name;
	  }
	  initFrameFile("maps/"+Buffer.frameFile+".framefile", type);
  }
  return type;
}

/*
 * places the party members on reloads and initial load.
 * called directly on initial party file load, but through
 * placeParty on subsequent map loads.
 */
private static void place(int i) {
  Buffer.Party.elementAt(i).x = 24*(Buffer.StartX+i/2);
  Buffer.Party.elementAt(i).y = 24*(Buffer.StartY+i%2);
}

/* On map reloads you don't want to reload all the stats
 * as it resets level advancements etc.
 * instead, we keep all the stats and just adjust the x/y
 * values of the party members. This is where placeParty
 * kicks in :)
 */
public static void placeParty() {
  for(int i=0;i<Buffer.Party.size();i++) {
         //Buffer.checkAlive(i);
         place(i);
      }
}

/*initItemFile is called by Shiny_Sword.class at the beginning of the game.*/
public static void initItemFile()
{
  Properties itemfile = Lib_Util.ReadProperties("gamefiles\\itemfile");

  boolean stop = false;
  int i = 0;
  while(!stop)
  {
   String wcallname = itemfile.getProperty(i+".wcallname","none");

   if(wcallname.equals("none"))
   {
    stop=true;
   }
   else if(wcallname==null)
   {
    stop=true;
   }
   else
   {

    Items h=new Items();
    h.wcallname=wcallname;
    h.wname = itemfile.getProperty(i+".wname","none");
    h.wtype = itemfile.getProperty(i+".wtype","none");
    h.wprice = Integer.parseInt(itemfile.getProperty(i+".wprice","none"));
    h.wimage = itemfile.getProperty(i+".wimage","none");
    h.damage = Integer.parseInt(itemfile.getProperty(i+".damage","none"));
    h.range = Integer.parseInt(itemfile.getProperty(i+".range","none"));

    Buffer.items.addElement(h);

    i++;

   }
  }
}

/*initShopFile is called by Shiny_Sword.class at the beginning of the game.*/
public static void initShopFile()
{
  Properties shopfile = Lib_Util.ReadProperties("gamefiles\\shopfile");

  boolean stop = false;
  int i = 0;
  while(!stop)
  {
   String sname = shopfile.getProperty(i+".sname","none");

   if(sname.equals("none"))
   {
    stop=true;
   }
   else if(sname==null)
   {
    stop=true;
   }
   else
   {

    Shops h=new Shops();
    h.shopformap=sname;
    h.shoptype = shopfile.getProperty(i+".shoptype","weapons");
    h.totalitems = Integer.parseInt(shopfile.getProperty(i+".totalitems"));

    //creates the new array of products
    h.addProducts(h.totalitems);

    //adds the values to the array of products
    int j = 0;
    while(j >= 0)
    {
     String temp = shopfile.getProperty(i+".product["+j+"]");
     if(temp != null)
     {
      h.product[j]=temp;
     }
     else
     {
      j = -2;
     }
     j++;
    }

    Buffer.shop.addElement(h);

    i++;

   }
  }
}

/* Initialize a party file by copying the contents of "partyfile"
 * to the "Party" Vector in the Buffer. The Party Vector serves
 * as a generic storage space for the characters
 * the itemfile is needed too.. */
public static void initPartyFile()
{
 /*Process:

 set up global variable gold.
 set up party vector, giving each member their startup items, whether they're in  the team
 their sprite, their portrait, their name, etc.
  */

  Properties entfile = Lib_Util.ReadProperties("gamefiles\\partyfile");
  Properties itemfile = Lib_Util.ReadProperties("gamefiles\\itemfile");

  Buffer.gold=Integer.parseInt(entfile.getProperty("goldstart"));

  boolean stop = false;
  int i = 1;
  while(!stop) {

    String source    = entfile.getProperty(i+".source","none");
    String join_later = entfile.getProperty(i+".joins_later","no");

    if(source.equals("none")) {
      stop = true;
      }
    else if(join_later.equals("no"))
    {
      /* Starting formation */
      //int tx           = Buffer.StartX+(i-1)/2;
      //int ty           = Buffer.StartY+(i-1)%2;


      String charname  = entfile.getProperty(i+".charname");
      String charclass = entfile.getProperty(i+".charclass");
      int type         = Integer.parseInt(entfile.getProperty(i+".type"));

      Entity e = new Entity(new Sprite(source),0,0);
      //place will set the right locations later on...

      int j = 0;

      while(j >= 0) {
      String temp = entfile.getProperty(i+".atkimage["+j+"]");
        if(temp != null) {
          e.addAtkImage(temp);
        }
        else {
          j = -2;
        }
        j++;
      }
      e.portrait=Lib_Image.getImage(entfile.getProperty(i+".portrait"));

      e.strength = Integer.parseInt(entfile.getProperty(i+".strength","0"));
      e.precision = Integer.parseInt(entfile.getProperty(i+".precision","0"));
      e.reflexes = Integer.parseInt(entfile.getProperty(i+".reflexes","0"));
      e.endurance = Integer.parseInt(entfile.getProperty(i+".endurance","0"));
      e.speed = Integer.parseInt(entfile.getProperty(i+".speed","0"));
      e.willpower = Integer.parseInt(entfile.getProperty(i+".willpower","0"));

      int ai = Integer.parseInt(entfile.getProperty(i+".ai","0"));
      if(ai>0) { e.ai = true; }
      else { e.ai = false;    }

      if(type == 0) {
        int level      = Integer.parseInt(entfile.getProperty(i+".level"));
        Buffer.Party.add(Stats.genCharStats(e,charname,charclass,level));


        //new stuff
        Buffer.Party.lastElement().item1 = entfile.getProperty(i+".item1","empty");
        Buffer.Party.lastElement().item2 = entfile.getProperty(i+".item2","empty");
        Buffer.Party.lastElement().item3 = entfile.getProperty(i+".item3","empty");
        Buffer.Party.lastElement().item4 = entfile.getProperty(i+".item4","empty");

        String weapon  = entfile.getProperty(i+".weapon","none");
        String findweapon;

        findweapon=entfile.getProperty(i+"."+weapon,"empty");

        int findWeapon=Buffer.findItem(findweapon);

        Buffer.Party.lastElement().damran = Buffer.items.elementAt(findWeapon).damage;
        Buffer.Party.lastElement().range = Buffer.items.elementAt(findWeapon).range;

        //need to change this so it gets it from the class, I'd say.
        Buffer.Party.lastElement().weptype = entfile.getProperty(i+".weptype","none");
        //end new stuff



      }
      if(type == 1) {

        Buffer.addEnt(Stats.genMonster(e,charclass));
      }

      /* Adjustments that affect both characters and monsters */
      String resp = entfile.getProperty(i+".responsive");
      if(resp != null)
        {

        String rtext = entfile.getProperty(i+".text");
        if(resp.equals("true") && rtext != null) {
            Buffer.Party.lastElement().responsive = true; Buffer.Party.lastElement().resptext = rtext;
            Buffer.Party.lastElement().respevent = entfile.getProperty(i+".textevent","none");
            //System.out.println(i+" has text: "+rtext);
          }

        }
    place(i-1);
    }
    i++;
  }
}
/* Initialize a entity file */
private static void initEntityFile(String name, int map_type)
{
  if(map_type == 0) { Buffer.resetToPartyOnly(); }
  else {
    Buffer.clearAllEnts();
    Buffer.addEnt(Buffer.Party.elementAt(0));
  }

  Properties entfile = Lib_Util.ReadProperties(name);
  Properties itemfile = Lib_Util.ReadProperties("gamefiles\\itemfile");

  boolean stop = false;
  int i = 1;
  while(!stop) {

    String source    = entfile.getProperty(i+".source","none");

    if(source.equals("none")) {
      stop = true;
      }
    else
    {
      int tx           = Integer.parseInt(entfile.getProperty(i+".tx"));
      int ty           = Integer.parseInt(entfile.getProperty(i+".ty"));
      String charname  = entfile.getProperty(i+".charname");
      String charclass = entfile.getProperty(i+".charclass");
      int type         = Integer.parseInt(entfile.getProperty(i+".type"));

      Entity e = new Entity(new Sprite(source),tx,ty);

      e.strength = Integer.parseInt(entfile.getProperty(i+".strength","0"));
      e.precision = Integer.parseInt(entfile.getProperty(i+".precision","0"));
      e.reflexes = Integer.parseInt(entfile.getProperty(i+".reflexes","0"));
      e.endurance = Integer.parseInt(entfile.getProperty(i+".endurance","0"));
      e.speed = Integer.parseInt(entfile.getProperty(i+".speed","0"));
      e.willpower = Integer.parseInt(entfile.getProperty(i+".willpower","0"));

      int ai = Integer.parseInt(entfile.getProperty(i+".ai","0"));
      if(ai>0) { e.ai = true; }
      else { e.ai = false;    }

      if(type == 0) {
        int level      = Integer.parseInt(entfile.getProperty(i+".level"));
        Buffer.addEnt(Stats.genCharStats(e,charname,charclass,level));
        String weapon  = entfile.getProperty(i+".weapon","none");
        Buffer.lastEnt().damran = 3;//Integer.parseInt(itemfile.getProperty(weapon+".damage","0"));
        Buffer.lastEnt().range = 1;//Integer.parseInt(itemfile.getProperty(weapon+".range","0"));

        /*
        //NEED TO APPLY THIS TO MONSTERS FOR ITEM DROPS
        Buffer.Party.lastElement().item1 = entfile.getProperty(i+".item1");
        Buffer.Party.lastElement().item2 = entfile.getProperty(i+".item2");
        Buffer.Party.lastElement().item3 = entfile.getProperty(i+".item3");
        Buffer.Party.lastElement().item4 = entfile.getProperty(i+".item4");
        //need to change this so it gets it from the class, I'd say.
        Buffer.Party.lastElement().weptype = entfile.getProperty(i+".weptype");
        //end new stuff
        */

        /* Add in attack animations for player characters */
        int j = 0;

        while(j >= 0) {
          String temp = entfile.getProperty(i+".atkimage["+j+"]");
          if(temp != null) {
            e.addAtkImage(temp);
          }
          else {
            j = -2;
          }
        j++;
        }
      }
      if(type == 1) {

        Buffer.addEnt(Stats.genMonster(e,charclass));
      }

      /* Adjustments that affect both characters and monsters */
      String resp = entfile.getProperty(i+".responsive");
      if(resp != null)
        {

        String rtext = entfile.getProperty(i+".text");
        if(resp.equals("true") && rtext != null) {
            Buffer.lastEnt().responsive = true; Buffer.lastEnt().resptext = rtext;
            Buffer.lastEnt().respevent = entfile.getProperty(i+".textevent","none");
            //System.out.println(i+" has text: "+rtext);
          }

        }

    }
    i++;
  }
}

/* Initialize a frame file - for movies */
private static void initFrameFile(String name, int map_type)
{
  Properties framefile = Lib_Util.ReadProperties(name);

  boolean stop = false;
  int i = 1;
    int totalFrames = Integer.parseInt(framefile.getProperty("totalframes"));
    Buffer.movieTicks=new int[totalFrames];
    Buffer.movieFrames=new String[totalFrames];

  while(!stop) {

    String source    = framefile.getProperty(i+".source","none");
    if(source.equals("none")) {
      stop = true;
     }
    else
    {
      String frameName = framefile.getProperty(i+".framename");
      String frameSource = framefile.getProperty(i+".source");
      int frameTicks = Integer.parseInt(framefile.getProperty(i+".frameticks"));
      Buffer.movieTicks[i-1]=frameTicks;
	  Buffer.movieFrames[i-1]=frameSource;
	  System.out.println("Here: "+Buffer.movieFrames[i-1]);
     }
    i++;
  }
}

/* Initialize a script file */
public static int initScriptFile(String name)
{
  Properties scrfile = Lib_Util.ReadProperties(name);

  Buffer.nextMap = scrfile.getProperty("general.nextmap");
  Buffer.currentMusic = scrfile.getProperty("general.music");
  Buffer.StartX = Integer.parseInt(scrfile.getProperty("general.startx","1"));
  Buffer.StartY = Integer.parseInt(scrfile.getProperty("general.starty","1"));
  Buffer.mapFile = scrfile.getProperty("general.mapfile");
  Buffer.entityFile = scrfile.getProperty("general.entityfile");
  Buffer.cur_script = scrfile;

  if(scrfile.getProperty("general.maptype") == null) {
    System.out.println("ERROR: map type seems to be undefined in the scriptfile:"+name);
    return(-1);
  }

  if(scrfile.getProperty("general.maptype").equals("movie"))
  { return(2); }

  if(scrfile.getProperty("general.maptype").equals("walkabout"))
  { return(1); }

  return(0);
}

/* Read a properties file. */
public static Properties ReadProperties(String name)
{
  Properties p = new Properties();
  try {
    p.load(new FileInputStream(name));
  } catch (IOException e) {
    System.out.println("Error reading properties file!");
  }
  return(p);
}


/* Write a properties file. */
public static void WriteProperties(Properties p, String name)
{
  try {
    p.store(new FileOutputStream(name),"This is a Shiny Sword Configuration File");
  } catch (IOException e) {
    System.out.println("Error writing properties file!");
  }
}

/* Converts Tile to SMF1 format (max 100000 tiles per tileset)*/
public static SMF_Tile1 Tile2SMF_Tile1(Tile t) {
	SMF_Tile1 smf = new SMF_Tile1();
	smf.nextTile = t.getNextTile() + (t.landEffect*100000);
	/* HACK: next tile is smf1 format stored land effect too! */
	smf.rgbs = t.getRgbs();
	return smf;
	}

/* Converts Tile from SMF1 format */
public static Tile SMF_Tile12Tile(SMF_Tile1 smf) {
	Tile t = new Tile();
	t.setRgbs(smf.rgbs);
	t.setNextTile(smf.nextTile%100000);
	t.landEffect = smf.nextTile/100000;
	return t;
	}

/* Converts Tileset to SMF1 format */
public static SMF_Tileset1 Tileset2SMF_Tileset1(Tileset t) {

	SMF_Tileset1 smf = new SMF_Tileset1();
	smf.tiles = new Vector<SMF_Tile1>();
	/* Generates a warning but so be it. There shall be no messing around
	 * with old classes ;) */

	smf.length = t.getLength();

	for(int i=0;i<t.getLength();i++) {
		smf.tiles.addElement(Tile2SMF_Tile1(t.getTile(i)));
		}

	return smf;
	}

/* Converts Tileset from SMF1 format */
public static Tileset SMF_Tileset12Tileset(SMF_Tileset1 smf) {
	Tileset t = new Tileset();

	/* Adjust size of the Tile vector */
	if(smf.length>t.getLength()) {
		for(int i=t.getLength();i<smf.length;i++) {
			t.addTile();
			}
		}
	/*Move tiles to the new Tile vector */
	for(int i=0;i<smf.length;i++) {
		t.setTile(i, SMF_Tile12Tile((SMF_Tile1) smf.tiles.elementAt(i)));
		}

	return t;
	}

/* Converts Map to SMF1 Format*/
public static SMF_WalkMap1 WalkMap2SMF_WalkMap1(WalkMap m) {
	SMF_WalkMap1 smf = new SMF_WalkMap1();
	smf.tileset = Tileset2SMF_Tileset1(m.getTileset());
	smf.tiles = m.getTiles();
	smf.width = m.getWidth();
	smf.height = m.getHeight();

	return smf;
	}

/* Converts Map from SMF1 Format*/
public static WalkMap SMF_WalkMap12WalkMap(SMF_WalkMap1 smf) {
	WalkMap m = new WalkMap();
	m.setTileset(SMF_Tileset12Tileset(smf.tileset));
	m.resize(smf.width, smf.height);
  m.setTiles(smf.tiles,smf.width,smf.height);

	return m;
	}
}
