import java.io.*;
import java.util.*;

/*
  Stats.java
  (c)1999-2005 SSC Development Team
  Function: Library of General Statistics Generation Functions.
  Allowed Dependencies: Entity.java
*/

public class Stats
{
  public static Entity genCharStats(Entity e, String name, String charclass, int level)
  {
   
    Properties classfile = Lib_Util.ReadProperties("gamefiles\\classfile");
    
    /* One attack animation for all at the moment */
    //String[] temp = {"images/codybattle1.gif","images/codybattle2.gif"};
    //e.addAtkImages(temp);
    
    e.name = name;
    e.classname = charclass;
    
    e.faction = 1; //for now, all the characters are on the good side
    
    int hp_base = Integer.parseInt(classfile.getProperty(charclass+".hp_base"));
    e.maxhp = hp_base + (e.endurance - 5)/2;
    e.curhp = e.maxhp;
    
    int mindam_base = Integer.parseInt(classfile.getProperty(charclass+".mindam_base"));
    e.mindam = mindam_base + (int) Math.round((e.strength-5)*0.2);
    
    int dodge_base = Integer.parseInt(classfile.getProperty(charclass+".dodge_base"));
    e.dod = dodge_base + (int) Math.round((e.reflexes-5)*0.5);
    
    int spellres_base = Integer.parseInt(classfile.getProperty(charclass+".spellres_base"));
    e.spellres = Math.max(spellres_base + (int) Math.round((e.willpower-5)*0.5),0);
    
    int mp_base = Integer.parseInt(classfile.getProperty(charclass+".mp_base","0"));
    if(mp_base > 0) {
      e.maxmp = mp_base + (e.willpower - 5)/2;
      e.curmp = e.maxmp;
    }  
    
    //spell name
    String sp_name = classfile.getProperty(charclass+".s0_name","none");
    //spell level
    int sp_lv = Integer.parseInt(classfile.getProperty(charclass+".s0_lv","1"));
    //level the spell is gained    
    int gained_at = Integer.parseInt(classfile.getProperty(charclass+".s0_gained_at","1"));
    //TODO: Add gaining spells at levelup!
    
    if(sp_name.equals("none") == false && gained_at == 1) {
      e.spell[0] = new Spell(sp_name, sp_lv);
    }
    
    sp_name = classfile.getProperty(charclass+".s1_name","none");
    sp_lv = Integer.parseInt(classfile.getProperty(charclass+".s1_lv","1"));
    gained_at = Integer.parseInt(classfile.getProperty(charclass+".s1_gained_at","1"));
    if(sp_name.equals("none") == false && gained_at == 1) {
      e.spell[1] = new Spell(sp_name, sp_lv);
    }
    
    sp_name = classfile.getProperty(charclass+".s2_name","none");
    sp_lv = Integer.parseInt(classfile.getProperty(charclass+".s2_lv","1"));
    gained_at = Integer.parseInt(classfile.getProperty(charclass+".s2_gained_at","1"));
    if(sp_name.equals("none") == false && gained_at == 1) {
      e.spell[2] = new Spell(sp_name, sp_lv);
    }
    
    sp_name = classfile.getProperty(charclass+".s3_name","none");
    sp_lv = Integer.parseInt(classfile.getProperty(charclass+".s3_lv","1"));
    gained_at = Integer.parseInt(classfile.getProperty(charclass+".s3_gained_at","1"));
    if(sp_name.equals("none") == false && gained_at == 1) {
      e.spell[3] = new Spell(sp_name, sp_lv);
    }
    
    int range = Integer.parseInt(classfile.getProperty(charclass+".range")); 
    e.range = range;
    
    int att_base = Integer.parseInt(classfile.getProperty(charclass+".att_base"));
    if(range>2) { e.att = att_base + e.precision - 5; }
    else {
      e.att = att_base + e.strength - 5;
    }
    
    int move = Integer.parseInt(classfile.getProperty(charclass+".move"));
    e.move = move + (e.speed-5)/4;
    
    //if(charclass.equals("Archer"))
    //{     
      //String[] archerimgs = {"images/archer01.gif","images/archer02.gif","images/archer03.gif"};
      //e.addAtkImages(archerimgs);
    //}
    
    e.level = 1;
    
    /* Levelling up to appopriate level */
    for(int i=1; i<level; i++) { e = levelupEntity(e); }
    
    return(e);
  }
  
  /* Quick Statistics function for fixed monsters */
  public static Entity genMonster(Entity e, String name)
  {
  //lizardman & cyclops!
    
    Properties mf = Lib_Util.ReadProperties("gamefiles\\monsterfile");

    e.name = name;
    e.faction = 2; //for now all monsters are on the evil side
    
    e.level = Integer.parseInt(mf.getProperty(name+".level"));
    
    e.exp = e.level*49;
    
    e.maxhp = Integer.parseInt(mf.getProperty(name+".hp"));
    e.curhp = e.maxhp;
    e.att = Integer.parseInt(mf.getProperty(name+".att"));
    e.mindam = Integer.parseInt(mf.getProperty(name+".mindam"));
    e.damran = Integer.parseInt(mf.getProperty(name+".damran"));
    e.dod = Integer.parseInt(mf.getProperty(name+".dodge"));
    e.spellres = Integer.parseInt(mf.getProperty(name+".spellres"));
    e.maxmp = Integer.parseInt(mf.getProperty(name+".mp"));
    e.curmp =e.maxmp;
    e.armor = Integer.parseInt(mf.getProperty(name+".armor"));
    e.range = Integer.parseInt(mf.getProperty(name+".range"));  
    e.move = Integer.parseInt(mf.getProperty(name+".move"));
    
    
    int j = 0;
      
    while(j >= 0) {
    String temp = mf.getProperty(name+".atkimage["+j+"]");
      if(temp != null) {
        e.addAtkImage(temp);
        //System.out.println(temp+" added!");
      }      
      else {
        j = -2;
      }
      j++;
    }
    
    /* One attack animation for all at the moment */
    //String[] temp = {"images/lizardman.gif","images/lizardman2.gif"};
    //e.addAtkImages(temp);
    
    //if(name.equals("Cyclops")) {
    //  String[] blae = {"images/cyclops1.gif","images/cyclops2.gif","images/cyclops3.gif"};
    //  e.addAtkImages(blae);
    //}  

    
    return(e);
  }
  
  public static int genIncrease(int s) 
  {
    int r = (int) (Math.random() * 12.0)+1;
    int i = 0;
    if(r == 1) { i = 0; }
    else if(r == 12) { i = s/10 + 2; }
    else if(r <= (s%10)+1) { i = s/10 + 1; }
    else { i = s/10; }
    
    if(i<0) { i = 0; }
    
    return i;
  }
  
  public static Entity levelupEntity(Entity e)
  {
    /* Adjust the experience */
    e.exp -= e.level*100;
    if(e.exp < 0) { e.exp = 0; } /* For forced level ups */
    
    e.level++;
    
    Properties cf = Lib_Util.ReadProperties("gamefiles\\classfile");
    
    int hp_score = Integer.parseInt(cf.getProperty(e.classname+".hp_score"));
    int inc = genIncrease(hp_score + e.endurance - 5);
    e.maxhp += inc;
    e.curhp += inc;
    
    int att_score = Integer.parseInt(cf.getProperty(e.classname+".att_score"));
    if(e.range > 2) { inc = genIncrease(att_score + e.precision - 5); }
    else { inc = genIncrease(att_score + e.strength - 5); }
    e.att += inc;
    
    int mindam_score = Integer.parseInt(cf.getProperty(e.classname+".mindam_score"));
    inc = genIncrease(mindam_score + (e.strength - 5)/5);
    e.mindam += inc;
    
    int dodge_score = Integer.parseInt(cf.getProperty(e.classname+".dodge_score"));
    inc = genIncrease(dodge_score + e.reflexes - 5);
    e.dod += inc;
    
    int spellres_score = Integer.parseInt(cf.getProperty(e.classname+".spellres_score"));
    inc = genIncrease(spellres_score + e.willpower - 5);
    e.spellres += inc;
    
    int mp_score = Integer.parseInt(cf.getProperty(e.classname+".mp_score"));
    if(mp_score > 0) {
      inc = genIncrease(mp_score + e.willpower - 5);
      e.maxmp += inc;
      e.curmp += inc;
    }  
      
    return(e);
  }  
  
  /* Combat calculation code
   * It's nice to have this stuff centrally. Makes it all the easier to edit ;).
   */
  
  
  /* TODO: implement fumbles and criticals! */
  public static char checkHit(Entity a, Entity d)
  {
    int hitback = isBackStab(a,d);
    
    //Backstab increases the effective attack chance by 5%
    if(Math.random()*100 > a.att+5*hitback-getTileMeleeMiss(d.x/24,d.y/24)) 
      { return('m'); } //Miss
    
    int superhit = 0;
    if(a.att > 100) { superhit = (a.att-100); }
    
    int dodge = (2*(d.dod - superhit) - hitback*d.dod)/2; //Dodge is halved with backstab
    
    if(Math.random()*100 < d.dod) return('d'); //Dodge
    
    if(hitback == 1) {
      return('b'); //Backstab Hit
    }
    return('h'); //Hit
  }
  
  /* TODO: implement special damage modifiers */
  public static int genDmg(Entity a, Entity d)
  {
    int hitback = isBackStab(a,d);
    
    /* Normal damage */
    int damage = ((a.mindam - d.armor) + 
                 (int) Math.round(Math.random()*(float)a.damran));
    
    if(hitback == 1) {
      damage = (int) Math.round((a.backstab*hitback*damage)/100.0);
    }
    
    if(damage<1) { damage = 1; }
    
    return damage;
  }  
  
  /* Check if attack is a backstab, return 1 if true */
  public static int isBackStab(Entity a, Entity d)
  {
    int face = d.f/2; /* 0 = up, 1 = right, 2 = down, 3 = left */
    
    if((a.y>d.y && face == 0)||(a.x<d.x && face == 1)||(a.y<d.y && face == 2)||(a.x>d.x && face == 3)) {
      return(1);
    }
    
    return(0);
  }
  
  /* Calculate exp gained for appopriate attack */
  public static int calcExp(Entity d, int damage, char type)
  {
    if(type == 'h' || type == 'b') {
      return (int) Math.max(Math.round((d.exp*damage)/(d.maxhp)),1);
    }
    if(type == 'k') {
      return Math.max(d.exp,1);
    }
    else { return(0); }
      
  }
  
  public static boolean checkLevelUp(Entity e) {
    if(e.exp >= e.level*100 && e.classname != null)
    {
      return(true); 
    }
    return(false);
  }

 
 
   
   
  /* LAND EFFECT CODE
   * This section is all about land effect, and the definition of the different
   * land types
   */ 
   
  
  private static String[] LEList     = {"Plain","Forest","Hills","Road","Desert","Solid","Ruins","Mountain","Roof"};
  /* TODO: make this part editable! */
  private static String[] LEBGs      = {"images/BGplains.png",
                                        "images/BGforest.png",
                                        "images/BGhill.png",
                                        "images/BGroad.png",
                                        "images/BGhill.png",
                                        "images/BGwall.png",
                                        "images/BGruins.png",
                                        "images/BGmountain.png",
                                        "images/BGplains.png"};
  private static int[] LEMeleeMiss   = {0       ,5       ,10     ,-5     ,10      ,0     ,0      ,20       ,0};
  private static int[] LEMissileMiss = {0       ,15      ,5      ,0      ,5       ,0     ,10     ,20       ,0};
  private static int[] LandMoveCost  = {11      ,13      ,18     ,10     ,18      ,1000  ,13     ,25       ,0};
  

  
  /* Getters */
  public static int getTileMeleeMiss(int x, int y)
  { return(LEMeleeMiss[getLE(getTileLE(x,y))]); }
  
  public static int getTileMissileMiss(int x, int y)
  { return(LEMeleeMiss[getLE(getTileLE(x,y))]); }
  
  public static int getTileMoveCost(int x,int y)
  { return(LandMoveCost[getLE(getTileLE(x,y))]); }
  
  public static String getTileBattleBackground(int x, int y)
  { return(LEBGs[getLE(getTileLE(x,y))]); }
  
  public static String[] getLEList()
  { return(LEList); }
  
  public static String getLE(int a) //get LE String from the list
  { return(LEList[a]); }
  
  public static String getTileLE(int x, int y)
  { 
    if(x<Buffer.getCurMap().getWidth() && y<Buffer.getCurMap().getHeight())
    {
      return(getLE(Buffer.getCurMap().getLE(x,y))); 
    }
    else {
      System.out.println("Error: the coordinates ("+x+","+y+") given to Stats.getTileLE are outside the map boundaries ("+Buffer.getCurMap().getWidth()+"x"+Buffer.getCurMap().getHeight()+").");
      return("Plain");
    }  
  }

  
  public static int getLE(String a)
  {
    boolean found = false;
    int i = 0;
    while(!found && i<LEList.length) {
      if(a.equals(LEList[i])) {
        return(i);
      }
      i++;
    }
    return(0);
  }
}
