import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/*
  WalkFrame.java
    (c)1999-2008 SSC Development Team
  Functions:
  -Managing the Game Walkabout GUI and input as a whole.
  Allowed Dependencies: WalkPanel.java
*/

public class WalkFrame  extends JFrame
            implements  ActionListener, KeyListener,
                  WindowFocusListener
{
  Container c = getContentPane();

  Timer timer;
  private JTextField prompt;

  WalkPanel screen;

  /* Value overview of directionals:
   *
   * 0 not moving up
   * 1 moving up, and the corresponding arrow key is still being pressed.
   * 2 moving up, but the key has been released.
   * 3 arrow key has been pressed, but other movement directions
   *   override this one
   */

  private int up = 0;
  private int down = 0;
  private int left = 0;
  private int right = 0;

  /*equals 1 if particular button was once pressed and not released yet.*/
  private int opup = 0;
  private int opdown = 0;
  private int opleft = 0;
  private int opright = 0;

  /* Mode var:
   * 0 = just walking around.
   * 1 = talking to nearest entity
   */
  private static int tempmode=0;
  public static int Tempmode() {
    return tempmode;
  }

  private static int mode = 0;
  public static int Mode() {
    return mode;
  }

  /* Flags Adder!! */
  private static String tempresp = "";
  public static String TempResp() {
    return tempresp;
  }

  /* Constructor */
  public WalkFrame(WalkMap cur_map)
  {
    this.setVisible(false);

    if(Buffer.playerIndex() > Buffer.numEnts()-1) { Buffer.firstPlayer(); }

    screen = new WalkPanel(cur_map,
    Buffer.Player().x,
    Buffer.Player().y);

    timer = new Timer(Buffer.TimerDelay, this); //Timer delay.TWEAKABLE
    prompt = new JTextField("");

    screen.setBounds(0,0,(int)(360*Buffer.ScaleRatio),(int)(240*Buffer.ScaleRatio));

    c.setLayout(null);

    c.add(screen);
    c.add(prompt);

    prompt.addKeyListener(this);
    this.addWindowFocusListener(this);

    this.setVisible(true);

    startAnimation();
  }

  /* initialization after map switches */
  public void init() {
    screen = new WalkPanel(Buffer.getCurMap(),Buffer.Player().x,Buffer.Player().y);

    Timer timer = new Timer(Buffer.TimerDelay, this); //Timer delay. TWEAKABLE
    JTextField prompt = new JTextField("");

    screen.setBounds(0,0,(int)(360*Buffer.ScaleRatio),(int)(240*Buffer.ScaleRatio));

    c.setLayout(null);

    c.add(screen);
    c.add(prompt);

    JukeBox.Init();

    prompt.addKeyListener(this);
    this.addWindowFocusListener(this);

    this.setVisible(true);

    JukeBox.PlayWalkTune();
    startAnimation();
  }

  /* Switch to a different map! */
  public void SwitchMap(String mapname)
  {
    stopAnimation();
    JukeBox.StopAll();
    this.setVisible(false);

    Buffer.firstPlayer();

    c.remove(screen);
    c.remove(prompt);

    Lib_Map.initMap(1,this,mapname);


  }

  public synchronized void startAnimation() {
    timer.start();
  }


  public synchronized void stopAnimation() {
    timer.stop();
  }

  private int anim_delay = Buffer.AnimDelay; //Animation delay per frame. TWEAKABLE
  private int anim_iterator = 0;

  /*
   * DRAWING ROUTINE
   */

  public int chk1(int a) //tiny math
  {
    if(a == 1) { return(1); }
    return(0);
  }

  public void actionPerformed(ActionEvent e) {

    //tile & sprite animation
    anim_iterator++;
    if(anim_iterator>=anim_delay) {
      anim_iterator = 0;
      Buffer.getCurMap().updateMap();
      Buffer.anim_toggle = 1 - Buffer.anim_toggle;

      /* Entity animation */
      if(Buffer.AnimateAllEnts == true) { //Animate all
        for(int i=0; i<Buffer.numEnts(); i++) {
          Buffer.getEnt(i).f = Buffer.getEnt(i).f - (Buffer.getEnt(i).f%2) + Buffer.anim_toggle;
        }
      }
      else { //Animate player only
        Buffer.Player().f = Buffer.Player().f - (Buffer.Player().f%2) + Buffer.anim_toggle;
      }
    }


    int newdirection = 0;

    //block next movement if obstructed!

    int ptx = Buffer.Player().x/24; //player tx
    int pty = Buffer.Player().y/24; //player ty
    int fac = Buffer.Player().faction;
    int spritenum = 0;
    int zonenum = 0;

      if(Buffer.Player().y%24 == 0) {

        zonenum = Buffer.getCurMap().getValue(ptx-chk1(up)+chk1(down),pty-chk1(left)+chk1(right),4);

        spritenum = Buffer.getCurMap().getValue(ptx,pty-1,5);
        if((Buffer.getCurMap().getValue(ptx,pty-1,3) == 1 ||
          spritenum > 0) && up == 1) {
          up = 0;
          opup = 1;
          newdirection = 1;
          //if(Buffer.Player().f/2 != 0) { Buffer.Player().f = 0 + Buffer.anim_toggle; }
        }
        spritenum = Buffer.getCurMap().getValue(ptx,pty+1,5);
        if((Buffer.getCurMap().getValue(ptx,pty+1,3) == 1 ||
          spritenum > 0) && down == 1) {
          down = 0;
          opdown = 1;
          newdirection = 1;
          //if(Buffer.Player().f/2 != 2) { Buffer.Player().f = 4 + Buffer.anim_toggle; }
        }
      }
      if(Buffer.Player().x%24 == 0) {

        zonenum = Buffer.getCurMap().getValue(ptx-chk1(up)+chk1(down),pty-chk1(left)+chk1(right),4);

        spritenum = Buffer.getCurMap().getValue(ptx+1,pty,5);
        if((Buffer.getCurMap().getValue(ptx+1,pty,3) == 1 ||
          spritenum > 0) && right == 1) {
          right = 0;
          opright = 1;
          newdirection = 1;
          //if(Buffer.Player().f/2 != 1) { Buffer.Player().f = 2 + Buffer.anim_toggle; }
        }
        spritenum = Buffer.getCurMap().getValue(ptx-1,pty,5);
        if((Buffer.getCurMap().getValue(ptx-1,pty,3) == 1 ||
          spritenum > 0) && left == 1) {
          left = 0;
          opleft = 1;
          newdirection = 1;
          //if(Buffer.Player().f/2 != 3) { Buffer.Player().f = 6 + Buffer.anim_toggle; }
        }
      }


    //actual movement
    int d = Buffer.PixelSkip;

    if(up==1 && Buffer.Player().y>1 )
      {
      Buffer.Player().y -= d; screen.Update(Buffer.Player().x,Buffer.Player().y);
      //if(Buffer.Player().f/2 != 0) { Buffer.Player().f = 0 + Buffer.anim_toggle; }
      }
    else if(down==1 && Buffer.Player().y < (Buffer.getCurMap().getHeight()*24)-24)
      {
      Buffer.Player().y += d; screen.Update(Buffer.Player().x,Buffer.Player().y);
      //if(Buffer.Player().f/2 != 2) { Buffer.Player().f = 4 + Buffer.anim_toggle; }
      }
    else if(left==1 && Buffer.Player().x>1)
      {
      Buffer.Player().x -= d; screen.Update(Buffer.Player().x,Buffer.Player().y);
      //if(Buffer.Player().f/2 != 3) { Buffer.Player().f = 6 + Buffer.anim_toggle; }
      }
    else if(right==1 && Buffer.Player().x < (Buffer.getCurMap().getWidth()*24)-24)
      {
      Buffer.Player().x += d; screen.Update(Buffer.Player().x,Buffer.Player().y);
      //if(Buffer.Player().f/2 != 1) { Buffer.Player().f = 2 + Buffer.anim_toggle; }
      }



    //finish old movement!


    else if(up==2 && Buffer.Player().y>1)
      {
      if(Buffer.Player().y%24 !=0) { Buffer.Player().y -= d; screen.Update(Buffer.Player().x,Buffer.Player().y); }
      else {
      up = 0;
      newdirection = 1;
      }
    }

    else if(down==2 && Buffer.Player().y < (Buffer.getCurMap().getHeight()*24)-24)
      {
      if(Buffer.Player().y%24 !=0) { Buffer.Player().y += d; screen.Update(Buffer.Player().x,Buffer.Player().y); }
      else {
      down = 0;
      newdirection = 1;
      }
    }

    else if(left==2 && Buffer.Player().x>1)
      {
      if(Buffer.Player().x%24 !=0) { Buffer.Player().x -= d; screen.Update(Buffer.Player().x,Buffer.Player().y); }
      else {
      left = 0;
      newdirection = 1;
      }
    }

    else if(right==2 && Buffer.Player().x < (Buffer.getCurMap().getWidth()*24)-24)
      {
      if(Buffer.Player().x%24 !=0) { Buffer.Player().x += d; screen.Update(Buffer.Player().x,Buffer.Player().y); }
      else {
      right = 0;
      newdirection = 1;
      }
    }

    else //if((up+1)/2 != 1 && (down+1)/2 != 1 && (left+1)/2 != 1 && (right+1)/2 != 1)
    {
     newdirection = 1;
    }

    if(newdirection == 1) {
      if     (up    == 3) { up    = 1; }
      else if(down  == 3) { down  = 1; }
      else if(left  == 3) { left  = 1; }
      else if(right == 3) { right = 1; }
    }

    //Map repaint
    screen.repaint();

    /* Zone Handling */
    Lib_Map.execZone(zonenum, this);
  }

  /*
   * KEYBOARD BUTTON HANDLING
   */

  public void keyPressed(KeyEvent e) {

    /* key becomes "VK_<something>" */
    int key = e.getKeyCode();

    /* Arrow keys initiate movement by queueing direction*/

    if(mode == 0) {

      /* Overland walking controls */
      if(key == e.VK_UP && up != 1) {
      /* "up != 1" prevents from queueing movement while the sprite is already
       * moving in the corresponding direction */
          up      = 3;
          opup    = 1;
          down    = 0;
          Buffer.Player().f = 0 + Buffer.anim_toggle;
        }
      else if(key == e.VK_DOWN && down != 1) {
          down    = 3;
          opdown  = 1;
          up      = 0;
          Buffer.Player().f = 4 + Buffer.anim_toggle;
        }
      else if(key == e.VK_LEFT && left != 1) {
          left    = 3;
          opleft  = 1;
          right   = 0;
          Buffer.Player().f = 6 + Buffer.anim_toggle;
        }
      else if(key == e.VK_RIGHT && right != 1) {
          right   = 3;
          opright = 1;
          left    = 0;
          Buffer.Player().f = 2 + Buffer.anim_toggle;
        }
      else if(key == e.VK_N) {
        SwitchMap(Buffer.nextMap);
      }
    }


    if(mode==1)
    {
//START ELVENS INSERTION MUAHAHAHAAAAA
    if(key == e.VK_UP)
    {

     if(Shop.menuplace==3)
     {
      Shop.buysell="BUY";
     }
     else if(Shop.menuplace==10 || Shop.menuplace==51)
     {
      if (Shop.memberPlace<=0)
      {
       Shop.memberPlace=0;
      }
      else
      {
       Shop.memberPlace=Shop.memberPlace-1;
      }
     }
     else if(Shop.menuplace==52)
     {
      if(Buffer.Party.elementAt(Shop.memberPlace).item1!="empty")
      {
       Shop.itemPlace=1;
      }
     }

    }
    else if(key == e.VK_DOWN)
    {

     if(Shop.menuplace==3)
     {
      Shop.buysell="SELL";
     }
     else if(Shop.menuplace==10 || Shop.menuplace==51)
     {
      if (Shop.memberPlace>=(Shop.partycount-1))
      {
       Shop.memberPlace=(Shop.partycount-1);
      }
      else
      {
       Shop.memberPlace=Shop.memberPlace+1;
      }
     }
     else if(Shop.menuplace==52)
     {
      if(Buffer.Party.elementAt(Shop.memberPlace).item4!="empty")
      {
       Shop.itemPlace=4;
      }
     }

    }
    else if(key == e.VK_LEFT)
    {

     if(Shop.menuplace==5)
     {
      if (Shop.squarePlace<=0)
      {
       Shop.squarePlace=0;
      }
      else
      {
       Shop.squarePlace=(Shop.squarePlace-1);
      }
     }
     else if(Shop.menuplace==6 || Shop.menuplace==11 || Shop.menuplace==12 || Shop.menuplace==13 || Shop.menuplace==53)
     {
      Shop.yesno="Yes";
     }
     else if(Shop.menuplace==52)
     {
      if(Buffer.Party.elementAt(Shop.memberPlace).item2!="empty")
      {
       Shop.itemPlace=2;
      }
     }

    }
    else if(key == e.VK_RIGHT)
    {

     if(Shop.menuplace==5)
     {
      if (Shop.squarePlace>=(Shop.weaponcount-1))
      {
       Shop.squarePlace=(Shop.weaponcount-1);
      }
      else
      {
       Shop.squarePlace=(Shop.squarePlace+1);
      }
     }
     else if(Shop.menuplace==6 || Shop.menuplace==11 || Shop.menuplace==12 || Shop.menuplace==13 || Shop.menuplace==53)
     {
      Shop.yesno="No";
     }
     else if(Shop.menuplace==52)
     {
      if(Buffer.Party.elementAt(Shop.memberPlace).item3!="empty")
      {
       Shop.itemPlace=3;
      }
     }

    }
    //END ELVENS INSERTION MUAHAHAAAA
   }




    /*
     * SSC is not a browser, but F5 does invoke a screen refresh.
     * Just in case really ;)
     */
    if(key == e.VK_F5)
    {
      this.paint(this.getGraphics());
    }
    //if(key == e.VK_ENTER)
    else if(key == e.VK_A || key == e.VK_D)
    {

      //ANOTHER OF ELVENS INSERTIONWS
      if(tempmode==1)
      {

       //intro -> buy/sell menu
       if(Shop.menuplace==2) { Shop.menuplace=3; }

       //buy/sell menu -> buy screen or sell screen
       else if(Shop.menuplace==3)
       {
        if(Shop.buysell=="BUY") { Shop.menuplace=5; }
        else { Shop.menuplace=50; }
       }

       //exit shop
       else if(Shop.menuplace==4) { mode=0; tempmode=0; }
       //weapon menu
       else if(Shop.menuplace==5) { Shop.yesno="Yes"; Shop.menuplace=6; }
       //yes or no for purchase
       else if(Shop.menuplace==6)
       {
         if(Shop.yesno=="Yes") { Shop.menuplace=9; }
         else { Shop.menuplace=7; }
      }
      // oh well you're missing out talk -> weapon menu
      else if(Shop.menuplace==7) { Shop.menuplace=5; }
      //you'll be happy with it talk ->weapon menu
      else if(Shop.menuplace==8) { Shop.menuplace=5; }
      //item chosen - text before member list.
      else if(Shop.menuplace==9) { Shop.menuplace=10; }

      //list members to choose who should hold item
      else if(Shop.menuplace==10)
      {
       //check if hands are full
       if(Buffer.Party.elementAt(Shop.memberPlace).item1.equals("empty")==true || Buffer.Party.elementAt(Shop.memberPlace).item2.equals("empty")==true || Buffer.Party.elementAt(Shop.memberPlace).item3.equals("empty")==true || Buffer.Party.elementAt(Shop.memberPlace).item4.equals("empty")==true)
       { //if so, then check if it's equippable
        if(Buffer.Party.elementAt(Shop.memberPlace).weptype.equals(Shop.wep[(Shop.squarePlace)].wtype))
        {
         Shop.yesno="Yes";
         Shop.menuplace=12;
        }
        else
        {
         Shop.yesno="Yes";
         Shop.menuplace=13;
        }
       }
       else
       {//no space in this guys inventory, wanna equip it to someone else?
        Shop.yesno="Yes";
        Shop.menuplace=11;
       }
      }

      //Oops your hands are full, anyone else?
      else if(Shop.menuplace==11)
      {
       if(Shop.yesno=="Yes") { Shop.menuplace=10; }
       else { Shop.menuplace=7; }
      }

      //Equip this item?
      else if(Shop.menuplace==12)
      {
       if(Shop.yesno=="Yes") { Shop.menuplace=8; }
       else { Shop.menuplace=8; }
      }

      //else - not equippable -> don't equip but give or reject
      else if(Shop.menuplace==13)
      {
        if(Shop.yesno=="Yes") { Shop.menuplace=8; }
        else { Shop.menuplace=7; }
      }

      //begin sell speech
      else if(Shop.menuplace==50) { Shop.menuplace=51; }

      //choosing between members
      else if(Shop.menuplace==51)
       {
        if(Buffer.Party.elementAt(Shop.memberPlace).item1=="empty" && Buffer.Party.elementAt(Shop.memberPlace).item2=="empty" && Buffer.Party.elementAt(Shop.memberPlace).item3=="empty" && Buffer.Party.elementAt(Shop.memberPlace).item4=="empty")
        { /*do nothing cos that member has no items*/ }
        else { Shop.menuplace=52; }
       }

      //choosing between items
      else if(Shop.menuplace==52) { Shop.yesno="Yes"; Shop.menuplace=53; }

      //choosing if you want to sell it for that price
      else if(Shop.menuplace==53)
      {
        if(Shop.yesno=="Yes") { Shop.menuplace=55; }
        else { Shop.menuplace=54; }
      }

      // item sold no -> begin sell speech
      else if(Shop.menuplace==54) { Shop.menuplace=50; }

      // item sold yes -> begin sell speech
      else if(Shop.menuplace==55) { Shop.menuplace=50; }


      } //END OF ELVENS INSERTION
      else
      {
       if(mode == 1)
       {
        mode = 0;
       }
        else if(mode == 0 && Buffer.DetectTalk()>=0)
       {
        mode = 1;
         /* Flags Adder (original) */
         //tempresp = Flags.Parse(Buffer.getEnt(Buffer.DetectTalk()).resptext);

         //another bit added by Elven, this turns tempmode on.
         String temp = (Buffer.getEnt(Buffer.DetectTalk()).resptext);
         if(temp.equals("shop") == true)
         {
          tempmode=1;
          Shop.menuplace=2;
         }
         /* Adjusted Flags Adder*/
         else {
           tempresp = Flags.Parse(Buffer.getEnt(Buffer.DetectTalk()).resptext);
         }
       }
      }
    }
    else if(key == e.VK_S) // Cancel button
    {
     //BEGIN ELVENS INSERTION

     if(tempmode==1)
     {
      //Useful but shallow routines for getting back in the shop system ;)
      if(Shop.menuplace==3) { Shop.menuplace=4; }
      if(Shop.menuplace==5 || Shop.menuplace==51) { Shop.menuplace=3; }
      else if(Shop.menuplace==6 || Shop.menuplace==10 || Shop.menuplace==11 || Shop.menuplace==13) { Shop.menuplace=7; }
      else if(Shop.menuplace==52) { Shop.menuplace=51; }
      else if(Shop.menuplace==53) { Shop.menuplace=54; }
     }
     //END ELVENS INSERTION
    }
    else if(key == e.VK_F && mode  == 0)
    {
      Buffer.Player().f = (Buffer.Player().f + 2)%8;
    }

  }

  public void keyTyped(KeyEvent e) { }
  public void keyReleased(KeyEvent e) {

    int key = e.getKeyCode();

    //Releasing the arrow keys stops any future movement in that direction,
    //but finishes the current move to the next tile

    if(key == e.VK_UP) {
      up = 2;
      opup = 0;
      if     (opdown  == 1) { down = 1; up    = 0; }
      else if(opright == 1) { left = 0; right = 3; }
      else if(opleft  == 1) { left = 3; right = 0; }
    }

    if(key == e.VK_DOWN) {
      down = 2;
      opdown = 0;
      if     (opup    == 1) { down = 0; up    = 1; }
      else if(opright == 1) { left = 0; right = 3; }
      else if(opleft  == 1) { left = 3; right = 0; }
    }

    if(key == e.VK_LEFT) {
      left = 2;
      opleft = 0;
      if     (opright == 1) { left = 0; right = 1; }
      else if(opup    == 1) { down = 0; up    = 3; }
      else if(opdown  == 1) { down = 3; up    = 0; }
    }

    if(key == e.VK_RIGHT) {
      right = 2;
      opright = 0;
      if     (opleft  == 1) { left = 1; right = 0; }
      else if(opup    == 1) { down = 0; up    = 3; }
      else if(opdown  == 1) { down = 3; up    = 0; }
    }

  }

  public void Refocus()
  {
    prompt.requestFocus();
  }

  /*Start animation routine on focus*/
  public void windowGainedFocus(WindowEvent e)
  {
    timer.start();
  }

  /*Halt animation routine on unfocus*/
  public void windowLostFocus(WindowEvent e) { timer.stop(); }
}

