import java.awt.*; import javax.swing.*; import java.io.*;
/*
	WalkMap.java 
	(c)1999-2003 SSC Development Team
	Function: Managing the area data structure
	Allowed Dependencies: Tileset.java, Tile.java	
*/

public class WalkMap implements Serializable
{
private static int[][] oldobs;
public static int[][] move_matrix;

//Tileset data structure containing the actual tile data 
private Tileset tileset;	
public Tileset getTileset() { return tileset; }
public void setTileset(Tileset t) { this.tileset = t; } 

//The grid which holds the tile values for every map square.
private int[][][] tiles;
public int[][][] getTiles() { return tiles; }
//public void setTiles(int[][][] t) { 
//  this.tiles = t;  
//}
public void setTiles(int[][][] t, int width, int height) { 
  this.tiles = new int[width][height][7];
  
  
  
  if(t[0][0].length == 6) {
    for(int i = 0; i < width; i++) {
      for(int j = 0; j < height; j++) {
        this.tiles[i][j][0] = t[i][j][0];
        this.tiles[i][j][1] = t[i][j][1];
        this.tiles[i][j][2] = t[i][j][2];
        this.tiles[i][j][3] = t[i][j][3];
        this.tiles[i][j][4] = t[i][j][4];
        this.tiles[i][j][5] = t[i][j][5];
        this.tiles[i][j][6] = 0;
      }    
    }
  }
  else {
    this.tiles = t;
  } 
}

//Map width/height in tiles.
private int width, height;
public int getHeight() { return height; }
public int getWidth() { return width; }

//References to the corresponding Script and EntityFile
private String Scriptfile, Entityfile;
public String getScriptfile() { return this.Scriptfile; }
public String getEntityfile() { return this.Entityfile; }
public void setScriptfile(String scr) { this.Scriptfile = scr; }
public void setEntityfile(String ent) { this.Entityfile = ent; }


public WalkMap() {
	
	}
	
public WalkMap(Tileset tileset, int width, int height) { //Constructor
	this.tiles = new int[width][height][7];
	this.oldobs = new int[width][height];
	this.tileset = tileset;
	this.width = width;		//width in tiles
	this.height = height;	//height in tiles
	
	for(int i=0;i<width;i++) {
		for(int j=0;j<height;j++) {
			tiles[i][j][0] = 1; //First layer
			tiles[i][j][1] = 0; //Second Layer
			tiles[i][j][2] = 0; //Third Layer
			tiles[i][j][3] = 0; //Obstruction/Land Effect Layer
			tiles[i][j][4] = 0; //Event Layer
			tiles[i][j][5] = 0; //Entity Layer
			
			/* SMF "2" extensions */
			tiles[i][j][6] = 0; //Fourth Layer
			}
		}
	}

/* Placement of movement obstacles */
public void placeMoveObs(Entity e) 
{
  move_matrix = Lib_Util.calcMoveArea(e);
  //mappings of relative matrix values to absolute tile coords
  int rsx = (e.x/24) - e.move - 1; 
  Buffer.MMorigX = rsx;
  int rsy = (e.y/24) - e.move - 1;
  Buffer.MMorigY = rsy;
  
  int mm_len = (e.move*2)+2;
  
  //Backup old obstructions
  //TODO: Tweak!
  this.oldobs = new int[width][height];
  for(int x=0; x<this.width; x++)
  {
    for(int y=0; y<this.height; y++)
    {
      oldobs[x][y] = tiles[x][y][3];
    }
  }
  
  //Debug stuff
  //System.out.println(this.width+" "+this.height+" "+oldobs.length+" "+tiles.length);
  
  for(int x=1; x<mm_len; x++)
  {
    //Border obs
    if(rsx+x>=0 && rsy>=0 && rsx+x<this.width && rsy<this.height) {
      tiles[rsx+x][rsy][3] = 1;
    }  
    if(rsx>=0 && rsy+x>=0 && rsx<this.width && rsy+x<this.height) {
      tiles[rsx][rsy+x][3] = 1;
    }
    if(rsx+x>=0 && rsy+mm_len>=0 && rsx+x<this.width && rsy+mm_len<this.height) {  
      tiles[rsx+x][rsy+mm_len][3] = 1;
    }
    if(rsx+mm_len>=0 && rsy+x>=0 && rsx+mm_len<this.width && rsy+x<this.height) {
      tiles[rsx+mm_len][rsy+x][3] = 1;
    }
    
    for(int y=1; y<mm_len; y++)
    {   
      if(move_matrix[x][y] == -1 && rsx+x>=0 && rsy+y>=0 && 
         rsx+x<this.width && rsy+y<this.height) {
        
        tiles[rsx+x][rsy+y][3] = 1;
      }
    }
  }   
}

public void restoreObs()
{
  for(int x=0; x<this.width; x++)
  {
    for(int y=0; y<this.height; y++)
    {
      tiles[x][y][3] = oldobs[x][y];
    }
  }
}


/* Getters! */
public Image getBlock(int x, int y, int layer) { //returns value of target Block on base layer. 
	return tileset.getTileContents(tiles[x][y][layer]);
	}

public int getValue(int x, int y, int layer) {
	
	//System.out.println("XYL: "+x+" "+y+" "+layer);
	
	if(x >= 0 && y >= 0 && x<width && y<height) { return tiles[x][y][layer]; }
	return -1; //error output
	}

public int getLE(int x, int y) {
  if(x<width && y<height) {
    return(tileset.getTile(tiles[x][y][0]).landEffect);
  }  
  else {
    System.out.println("Error: the coordinates given to WalkMap.getLE are outside the map boundaries.");
    return(0);
  }
}	

public boolean detectRoof(int x, int y) {
  if((tileset.getTile(tiles[x][y][2]).landEffect) == 8) {
    return true;
  }
  return false;
}
	
public void setBlock(int x, int y, int layer, int value) { //sets value of target block.
	if(x >= 0 && y >= 0 && x<width && y<height) {
		tiles[x][y][layer] = value;
		}
	}	

public void setValue(int x, int y, int layer, int value) { //purely for my convenience ;).
    setBlock(x, y, layer, value);
  }

public void updateMap() { 
//Updates the map animation
	for(int i=0;i<width;i++) {
	for(int j=0;j<height;j++) {
			int next0 = tileset.getTile(this.getValue(i,j,0)).getNextTile();
			int next1 = tileset.getTile(this.getValue(i,j,1)).getNextTile();
			int next2 = tileset.getTile(this.getValue(i,j,2)).getNextTile();
      int next3 = tileset.getTile(this.getValue(i,j,6)).getNextTile();
			if(next0 > 0) { tiles[i][j][0] = next0; }
			if(next1 > 0) { tiles[i][j][1] = next1; }
			if(next2 > 0) { tiles[i][j][2] = next2; }
      if(next3 > 0) { tiles[i][j][6] = next3; }
			}
		}
	}
	
public void resize(int w, int h) {		
	int[][][] temp = this.tiles;
	this.tiles = new int[w][h][7];
	
	for(int i=0;i<w && i<width;i++) {
	for(int j=0;j<h && j<height;j++) {
		tiles[i][j][0] = temp[i][j][0];
		tiles[i][j][1] = temp[i][j][1];
		tiles[i][j][2] = temp[i][j][2];
    tiles[i][j][3] = temp[i][j][3];
    tiles[i][j][4] = temp[i][j][4];
    tiles[i][j][5] = temp[i][j][5];
    tiles[i][j][6] = temp[i][j][6];
		}
	}
	
	this.width = w;
	this.height = h;
	}
}